package cn.cloudcore.iprotect.plugin;

import java.util.UUID;

import android.app.Activity;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.ContextWrapper;
import android.content.Intent;
import android.content.IntentFilter;
import android.graphics.Color;
import android.graphics.Rect;
import android.os.Bundle;
import android.support.v4.content.LocalBroadcastManager;
import android.text.InputType;
import android.util.AttributeSet;
import android.util.Log;
import android.view.ContextMenu;
import android.view.KeyEvent;
import android.view.MotionEvent;
import android.view.View;
import android.view.WindowManager;
import android.view.inputmethod.InputMethodManager;
import android.widget.EditText;

public class CEditText extends EditText {
    
    private String CKBD_RECEIVED = "CKBD.ACTION_EDIT.HANDLE";
    private String CET_RECEIVED = "CET.ACTION_EDIT.HANDLE";
    
    private Context cETContext;
    private int  softInputMode;

    private CEditTextReceiver infoReceiver = null;
    
    private CKbdJniLib ckbd = null;
    private String cEditTextName = null;
    private String maskStr = "*";
    private int textColor = Color.BLACK;
    private boolean vibrator = false;
    private boolean initial = false;
    private short softkbdView = CEditTextAttrSet.SOFT_KBD_NO_EDITTEXT;
    private short softkbdType = CEditTextAttrSet.SOFT_KBD_QWERTY;
    private boolean clearWhenOpenKbd = true;
    private boolean clearWhenOnPause = true;
    private short finishMode = CEditTextAttrSet.SOFT_KBD_FINISH_MODE_CLOSE;
    private boolean popMode = true;
    
    private boolean isZoomOut = false;
    private float   zoomOutScaleX = 2.0f / 3;
    private float   zoomOutScaleY = 2.0f / 3;

    public CEditText(Context context) {
        super(context);
        cETContext = context;
    }
    
    public CEditText(Context context, AttributeSet attrs) {
        super(context, attrs);
        cETContext = context;
    }

    public CEditText(Context context, AttributeSet attrs, int defStyle) {
        super(context, attrs, defStyle);
        cETContext = context;
    }

    private void hideSystemInput() {
        this.setOnTouchListener(new View.OnTouchListener() {
            
            public boolean onTouch(View view, MotionEvent event) {
                int inType = CEditText.this.getInputType(); 
                CEditText.this.setInputType(InputType.TYPE_NULL);
                CEditText.this.onTouchEvent(event);
                CEditText.this.setInputType(inType);

                return true;
            }
        });
    }
    
    public boolean initialize(CEditTextAttrSet attr) {
        
        if (attr == null) return false;
        
        if (initial == true) return true;
        
//        CEditTextAttrSet tmpSet = new CEditTextAttrSet();
//        if (attr.name.equalsIgnoreCase(tmpSet.name)) {
//            UUID uuid = UUID.randomUUID();
//            cEditTextName = uuid.toString();    
//        } else {
//            cEditTextName = attr.name;
//        }
        
        UUID uuid = UUID.randomUUID();
        cEditTextName = uuid.toString();
        ckbd = new CKbdJniLib(cEditTextName);
        
        this.setSingleLine();
        this.setTextColor(attr.textColor);
        
        if (attr.switchMode == CEditTextAttrSet.SOFT_KBD_SWITCH_BETWEEN_ALPHA_NUMBER) {
            if (attr.softkbdType == CEditTextAttrSet.SOFT_KBD_MONEY) {
                attr.softkbdType = CEditTextAttrSet.SOFT_KBD_NUMBER;
            }
        }
        
        ckbd.setKbdType(attr.softkbdType);
        ckbd.setKbdStyle(attr.softkbdStype);
        ckbd.setAccepts(attr.accepts);
        ckbd.setMaxLength(attr.maxLength);
        ckbd.setMinLength(attr.minLength);
        ckbd.setKbdRandom(attr.kbdRandom);
        ckbd.setKbdRandomType(attr.kbdRandomType);
        ckbd.setContentType(attr.contentType);
        //ckbd.setKbdMode(attr.softkbdMode);
        if (attr.softkbdMode == CEditTextAttrSet.SOFT_KBD_MODE_KDOWN_NOPOP) {
            popMode = false;
            ckbd.setKbdMode(CEditTextAttrSet.SOFT_KBD_MODE_KDOWN_CHANGE);
        } else {
            ckbd.setKbdMode(attr.softkbdMode);
        }
        //ckbd.setAdjustTime(attrset.adjustTime);
        ckbd.setSwitchMode(attr.switchMode);
        ckbd.setFinishMode(attr.finishMode);
        
        //cEditTextName = attr.name;
        textColor = attr.textColor;
        vibrator = attr.kbdVibrator;
        softkbdView = attr.softkbdView;
        softkbdType = attr.softkbdType;
        clearWhenOpenKbd = attr.clearWhenOpenKbd;
        clearWhenOnPause = attr.clearWhenOnPause;
        finishMode = attr.finishMode;
        maskStr = String.format("%c", attr.maskChar);
        
        isZoomOut = attr.isZoomOut;
        zoomOutScaleX = attr.zoomOutScaleX;
        zoomOutScaleY = attr.zoomOutScaleY;
        
        onSetContentWithMask();
        
        hideSystemInput();
        initial = true;
        
        return true;
    }
    
    
    public void onDestroy() {
        if (infoReceiver != null) {
            LocalBroadcastManager lbm = LocalBroadcastManager.getInstance(cETContext);
            lbm.unregisterReceiver(infoReceiver);
        }
        
        infoReceiver = null;
        ckbd.destroy();
        
        initial = false;
    }
    
    private void onStartActivity() {
        if (infoReceiver != null) return;

        String receiverHandle = CET_RECEIVED + "_" + ckbd.kbd_handle;
        infoReceiver = new CEditTextReceiver();
        IntentFilter filter=new IntentFilter();
        
        filter.addAction(receiverHandle);
        
        LocalBroadcastManager lbm = LocalBroadcastManager.getInstance(cETContext);
        lbm.registerReceiver(infoReceiver,  filter);
    }
    
    private void onCloseActivity() {
        if (infoReceiver == null) return;
        
        LocalBroadcastManager lbm = LocalBroadcastManager.getInstance(cETContext);
        lbm.unregisterReceiver(infoReceiver);
        infoReceiver = null;
    }
    
    @Override
    public void setHighlightColor(int color) {
        super.setHighlightColor(color);
    }
    
    @Override
    protected void onCreateContextMenu(ContextMenu menu) {
        return;
    }
    
    @Override
    public boolean onKeyDown(int keyCode, KeyEvent event) {
        if (keyCode == KeyEvent.KEYCODE_BACK || keyCode == KeyEvent.KEYCODE_HOME
                || keyCode == KeyEvent.KEYCODE_MENU || keyCode == KeyEvent.KEYCODE_SEARCH) {
            return false;
        }
        
        return true;
    }
    
    private Activity getActivity(Context context) {
        while (!(context instanceof Activity) && context instanceof ContextWrapper) {
            context = ((ContextWrapper) context).getBaseContext();
        }

        if (context instanceof Activity) {
            return (Activity) context;
        }
    
        return (Activity) context;
    }
    
    @Override
    public void onFocusChanged(boolean focused, int direction, 
            Rect previouslyFocusedRect) {
        InputMethodManager imm;
        //String hint;
        
        if (focused == true) {
            
            Activity activity = getActivity(cETContext);
            if (activity == null) return;
            
            softInputMode = activity.getWindow().getAttributes().softInputMode;
            
            imm = (InputMethodManager)activity.getSystemService(Context
                    .INPUT_METHOD_SERVICE);
            imm.hideSoftInputFromWindow(this.getWindowToken(), 0);
            this.setCursorVisible(true);
            
            
            this.requestFocus();
            Intent intent = new Intent(cETContext, CKbdActivity.class);
            intent.putExtras(createKbdAttrSet());
            cETContext.startActivity(intent);
            
            if (clearWhenOpenKbd == true)
                ckbd.clearPlaintext();
            
            onSetContentWithMask();
            onStartActivity();
            this.setSelection(this.length());
            
            
        } else {
            imm = (InputMethodManager)cETContext.getSystemService(Context.
                    INPUT_METHOD_SERVICE);
            imm.hideSoftInputFromWindow(getWindowToken(), InputMethodManager.
                    HIDE_NOT_ALWAYS);
            setCursorVisible(false);
            
//            ((Activity)cETContext).getWindow().setSoftInputMode(softInputMode | 
//                    WindowManager.LayoutParams.SOFT_INPUT_IS_FORWARD_NAVIGATION);
            ((Activity)cETContext).getWindow().setSoftInputMode(softInputMode);
        }
    }
    
    @Override
    protected void onVisibilityChanged(View changedView, int visibility) {        
        
        if (visibility == View.INVISIBLE && this.hasFocus()) {
            ((Activity)cETContext).getWindow().setSoftInputMode(WindowManager
                    .LayoutParams.SOFT_INPUT_STATE_UNCHANGED);
        }
        
        super.onVisibilityChanged(changedView, visibility);
    }
    
    
    
    @Override
    public boolean onTouchEvent(MotionEvent event) {
        
        if (initial == false) return true;
        
        switch (event.getAction()) {    
            case MotionEvent.ACTION_UP:
                this.requestFocus();
                Intent intent = new Intent(cETContext, CKbdActivity.class);
                intent.putExtras(createKbdAttrSet());
                cETContext.startActivity(intent);
                
                if (clearWhenOpenKbd == true)
                    ckbd.clearPlaintext();
                
                onSetContentWithMask();
            
                onStartActivity();
                break;
        }

        super.onTouchEvent(event);
        this.setSelection(this.length());

        return true;
    }
    
    private Bundle createKbdAttrSet() {
        Bundle bundle = new Bundle();
        
        bundle.putString("cEditTextName", cEditTextName);
        bundle.putBoolean("kbdVibrator", vibrator);
        bundle.putString("maskStr", maskStr);
        bundle.putInt("textColor", textColor);
        bundle.putShort("softkbdView", softkbdView);
        bundle.putShort("softkbdType", softkbdType);
        bundle.putShort("finishMode", finishMode);
        bundle.putBoolean("popMode", popMode);
        
        bundle.putBoolean("isZoomOut", isZoomOut);
        bundle.putFloat("zoomOutScaleX", zoomOutScaleX);
        bundle.putFloat("zoomOutScaleY", zoomOutScaleY);
        bundle.putBoolean("clearWhenOnPause", clearWhenOnPause);
        
        
        return bundle;
    }
        
    private void onSetContentWithMask() {
        String str="";
        for (int i = 0; i < ckbd.getPlaitextLen(); i++) {
            str = str+maskStr;
        }

        this.setText(str);
        this.setSelection(this.length());
    }
    
    private void openSystemSoftKeyboard() {
        
        InputMethodManager imm = (InputMethodManager) cETContext.getSystemService(Context.INPUT_METHOD_SERVICE);
        
        this.requestFocus();
        //imm.showSoftInput(this, 0);
        imm.showSoftInput(this, InputMethodManager.SHOW_FORCED);

    }
    
    private class CEditTextReceiver extends BroadcastReceiver {
        String receiverHandle = CET_RECEIVED + "_" + ckbd.kbd_handle;
        String updateInfo = "UpdateInfo";
        String closeInfo  = "CloseInfo";
        String openInfo = "OpenInfo";
        String cKbdInfo;
        
        int kbdHandle;
        
        @Override
        public void onReceive(Context context, Intent intent) {

            if (intent.getAction().equals( receiverHandle )) {
                kbdHandle = intent.getIntExtra("CKbdHandle", -1);

                if (kbdHandle == ckbd.getNativeHandle()) {
                    cKbdInfo = intent.getStringExtra("CKbdInfo");
                    if (cKbdInfo == null) return;
                    
                    if (cKbdInfo.toLowerCase().compareTo(updateInfo.toLowerCase()) == 0) {
                            onSetContentWithMask();
                    }
                    
                    if (cKbdInfo.toLowerCase().compareTo(closeInfo.toLowerCase()) == 0) {
                        onSetContentWithMask();
                        
                        if (finishMode == CEditTextAttrSet.SOFT_KBD_FINISH_MODE_CLOSE) 
                            onCloseActivity();
                        
                    }
                    
                    if (cKbdInfo.toLowerCase().compareTo(openInfo.toLowerCase()) == 0) {
                        //openSystemSoftKeyboard();
                    }
                }
            }
        }
    }
    
    public String getValue(String timeStamp) {
        if (initial == false) return null;
        
        return ckbd.getCiphertext(timeStamp);
    }
    
    public String getPinValue(String timeStamp) {
        if (initial == false) return null;

        return ckbd.getPinCiphertext(timeStamp);
    }
    
    public boolean publicKeyModulus(String keyModulus) {
        if (initial == false) return false;
        
        return ckbd.publicKeyModulus(keyModulus);
    }
    
    public boolean publicKeyDER(String keyDer) {
        if (initial == false) return false;
        
        return ckbd.publicKeyDER(keyDer);
    }
    
    public boolean publicKeyBlob(String keyBlob) {
        if (initial == false) return false;
        
        return ckbd.publicKeyBlob(keyBlob);
    }
    
    public boolean publicKeyAppModulus(String keyModulus) {
        if (initial == false) return false;
        
        return ckbd.publicKeyAppModulus(keyModulus);
    }
    
    public boolean publicKeyAppDER(String keyDer) {
        if (initial == false) return false;
        
        return ckbd.publicKeyAppDER(keyDer);
    }
    
    public boolean publicKeyAppBlob(String keyBlob) {
        if (initial == false) return false;
        
        return ckbd.publicKeyAppBlob(keyBlob);
    }
    
    public boolean publicKeyECC(String x,String y) {
        return ckbd.publicKeyECC(x, y);
    }

    public short verify() {
        if (initial == false) return -1;
        
        return ckbd.verify();
    }
    public void clear() {
        if (initial == false) return;
        
        ckbd.clearPlaintext();
        this.setText("");
    }
    
    public short getLength() {
        if (initial == false) return 0;
        
        return ckbd.getPlaitextLen();
    }
    
    public long getVersion() {
        if (initial == false) return 0;
        
        return ckbd.getVersion();
    }
    
    public char getComplexDegree() {
        if (initial == false) return 0;
        
        return ckbd.getComplexDegree();
    }
    
    public String lastError() {
        if (initial == false) return null;
        
        return ckbd.lastError();
    }
    
    public void openCKbd() {
        if (initial == false) {
            Log.e("DEBUG_INFO", "not initialize!");
            return;
        }
        
        Intent intent = new Intent(cETContext, CKbdActivity.class);
        intent.putExtras(createKbdAttrSet());
        cETContext.startActivity(intent);
        onSetContentWithMask();
        
        onStartActivity();
    }
    
    public void closeCKbd() {
        if (initial == false) return;

        Intent intent = new Intent();
        String receiverHandle = CKBD_RECEIVED + "_" + ckbd.kbd_handle;
        intent.setAction(receiverHandle);
        intent.putExtra("CKbdHandle", ckbd.kbd_handle);
        intent.putExtra("CKbdCommand", "close");

        LocalBroadcastManager lbm = LocalBroadcastManager.getInstance(cETContext);
        lbm.sendBroadcast(intent);
        
        onCloseActivity();
    }
    
    public void setChallengeCode(byte[] code) {
        if (initial == false) return;
        
        ckbd.setChallengeCode(code);
    }
    
    public byte[] getEncryptedPinCode() {
        if (initial == false) return null;
        
        return ckbd.getEncryptedPinCode();
    }
    
    public void setCalcFactor(String factor) {
        if (initial == false) return;
        
        ckbd.setCalcFactor(factor);
    }
    
    public void setContentType(int contentType) {
        if (initial == false) return;
        
        ckbd.setContentType(contentType);
    }
    
    public void setSoftkbdType(short softkbdType) {
        if (initial == false) return;
        
        ckbd.setKbdType(softkbdType);
    }
    
    public void setSoftkbdStype(short softkbdStype) {
        if (initial == false) return;
        
        ckbd.setKbdStyle(softkbdStype);
    }
    
    public void setSoftkbdMode(short softkbdMode) {
        if (initial == false) return;
        
        ckbd.setKbdMode(softkbdMode);
    }
    
    public void setAccepts(String accepts) {
        if (initial == false) return;
        
        ckbd.setAccepts(accepts);
    }
    
    public void setMaxLength(short maxlength) {
        if (initial == false) return;
        
        ckbd.setMaxLength(maxlength);
    }
    
    public void setMinLength(short minlength) {
        if (initial == false) return;
        
        ckbd.setMinLength(minlength);
    }
    
    public void setMode(short mode) {
        if (initial == false) return;
        
        ckbd.setMode(mode);
    }
    
    public void setAlgorithmCode(String algorithmCode) {
        ckbd.setAlgorithmCode(algorithmCode);
    }
    
    public void setHashRandom(String salt) {
        ckbd.setHashRandom(salt);
    }
    
    public void setDictionaryFilter(String dictFilter) {
        ckbd.setDictionaryFilter(dictFilter);
    }
    
    public String getReceiverAction() {
        return CET_RECEIVED + "_" + ckbd.kbd_handle;
    }
    
    public String getMeasureValue() {
        return ckbd.getMeasureValue();
    }
    
    public void setFinishMode(short finishMode) {
        this.finishMode = finishMode;
        ckbd.setFinishMode(finishMode);
    }
    
    public void setZoomOut(float zoomOutX, float zoomOutY) {
        isZoomOut = true;
        zoomOutScaleX = zoomOutX;
        zoomOutScaleY = zoomOutY;
    }
}
