package cn.cloudcore.iprotect.plugin;


import android.app.Activity;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.Bundle;
import android.support.v4.content.LocalBroadcastManager;

public class CKeyBoard {
    
    private String CKBD_RECEIVED = "CKBD.ACTION_EDIT.HANDLE";
    private String CET_RECEIVED  = "CET.ACTION_EDIT.HANDLE";
    
    private CKbdJniLib ckbd = null;
    private CEditTextAttrSet attrSet;
    private boolean popMode = true;
    
    private CEditTextReceiver infoReceiver = null;
    private boolean initial = false;
    private String cKbdName = null;
    
    private Context context;
    
    private CKeyBoardStateInterface  ckbdStateInterface = null;
    private String                   ckbdStateFuncName = null;
    
    public CKeyBoard(Context context, String cKbdName) {
        this.context = context;
        this.cKbdName = cKbdName;
        
        ckbd = new CKbdJniLib(cKbdName);
    }
    
    public boolean initCSoftKeyBoard(CEditTextAttrSet attr) {
        if (attr == null) return false;
        
        attrSet = attr;
        attrSet.name = cKbdName;
        
        ckbd.setKbdType(attr.softkbdType);
        ckbd.setKbdStyle(attr.softkbdStype);
        ckbd.setAccepts(attr.accepts);
        ckbd.setMaxLength(attr.maxLength);
        ckbd.setMinLength(attr.minLength);
        ckbd.setKbdRandom(attr.kbdRandom);
        ckbd.setKbdRandomType(attr.kbdRandomType);
        ckbd.setContentType(attr.contentType);
        ckbd.setKbdMode(attr.softkbdMode);
        ckbd.setSwitchMode(attr.switchMode);
        initial = true;
        
        return true;
    }
    
    private void onStartActivity() {
        if (infoReceiver != null) return;

        String receiverHandle = CET_RECEIVED + "_" + ckbd.kbd_handle;
        infoReceiver = new CEditTextReceiver();
        IntentFilter filter=new IntentFilter();
        
        filter.addAction(receiverHandle);
        
        LocalBroadcastManager lbm = LocalBroadcastManager.getInstance(context);
        lbm.registerReceiver(infoReceiver, filter);
    }
    
    private void onCloseActivity() {
        if (infoReceiver == null) return;
        
        LocalBroadcastManager lbm = LocalBroadcastManager.getInstance(context);
        lbm.unregisterReceiver(infoReceiver);
        infoReceiver = null;
    }
    
    public Bundle createKbdAttrSet() {
        Bundle bundle = new Bundle();

        bundle.putString("cEditTextName", attrSet.name);
        bundle.putBoolean("kbdVibrator", attrSet.kbdVibrator);
        bundle.putString("maskStr", attrSet.maskChar+"");
        bundle.putInt("textColor", attrSet.textColor);
        bundle.putShort("softkbdView", attrSet.softkbdView);
        bundle.putShort("softkbdType", attrSet.softkbdType);
        bundle.putBoolean("popMode", popMode);
        bundle.putBoolean("isZoomOut", attrSet.isZoomOut);
        bundle.putFloat("zoomOutScaleX", attrSet.zoomOutScaleX);
        bundle.putFloat("zoomOutScaleY", attrSet.zoomOutScaleY);
        
        
        return bundle;
    }
    
    public void openCKbd() {
        if (initial == false) return;
        
        Intent intent = new Intent(context, CKbdActivity.class);
        intent.putExtras(createKbdAttrSet());
        context.startActivity(intent);
        
        onStartActivity();
    }
    
    public void openCkbdForPay(Activity activity, int requestCode) {
        Intent intent = new Intent();
        intent.setClass(activity, CKbdForPayActivity.class);
        intent.putExtras(createKbdAttrSet());
        
        activity.startActivityForResult(intent, requestCode);
    }
    
    public void closeCKbd() {
        if (initial == false) return;

        Intent intent = new Intent();
        String receiverHandle = CKBD_RECEIVED + "_" + ckbd.kbd_handle;
        intent.setAction(receiverHandle);
        intent.putExtra("CKbdHandle", ckbd.kbd_handle);
        intent.putExtra("CKbdCommand", "close");

        LocalBroadcastManager lbm = LocalBroadcastManager.getInstance(context);
        lbm.sendBroadcast(intent);
        
        onCloseActivity();
    }
    
    public void destroy() {
        ckbd.destroy();
    }
    
//    public void setUpdateInterface(CKeyBoardUpdateInterface cInterface) {
//        ckbdUpInterface = cInterface;
//    }
//      
//    public void setWebViewUpdateFuncName(String name) {
//        ckbdUpFuncName = name;
//    }
    
    public void setStateInterface(CKeyBoardStateInterface cKeyBoardStateInterface) {
        this.ckbdStateInterface = cKeyBoardStateInterface;
    }
    
    public void setWebViewStateFuncName(String name) {
        ckbdStateFuncName = name;
    }
    
    private class CEditTextReceiver extends BroadcastReceiver {
        String receiverHandle = CET_RECEIVED + "_" + ckbd.kbd_handle;
        String updateInfo = "UpdateInfo";
        String closeInfo  = "CloseInfo";
        String openInfo = "OpenInfo";
        
        String cKbdInfo;
        
        int kbdHandle;
        
        @Override
        public void onReceive(Context context, Intent intent) {

            if (intent.getAction().equals( receiverHandle )) {
                kbdHandle = intent.getIntExtra("CKbdHandle", -1);
                if (kbdHandle == ckbd.getNativeHandle()) {
                    cKbdInfo = intent.getStringExtra("CKbdInfo");
                    if (cKbdInfo == null) return;
                    
                    if (cKbdInfo.toLowerCase().compareTo(updateInfo.toLowerCase()) == 0) {
                    }
                    
                    if (cKbdInfo.toLowerCase().compareTo(closeInfo.toLowerCase()) == 0) {
                        onCloseActivity();
                        
                        
                    }
                    
                    if (cKbdInfo.toLowerCase().compareTo(openInfo.toLowerCase()) == 0) {
                        onStartActivity();
                    }
                    
                    if (ckbdStateInterface != null) {
                        ckbdStateInterface.receive(ckbdStateFuncName, cKbdName, cKbdInfo, ckbd.getPlaitextLen());
                    }
                }
            }
        }
    }
    
    public String getValue(String timeStamp) {
        if (initial == false) return null;
        
        return ckbd.getCiphertext(timeStamp);
    }
    
    public String getPinValue(String timeStamp) {
        if (initial == false) return null;

        return ckbd.getPinCiphertext(timeStamp);
    }
    
    public boolean publicKeyModulus(String keyModulus) {
        if (initial == false) return false;
        
        return ckbd.publicKeyModulus(keyModulus);
    }
    
    public boolean publicKeyDER(String keyDer) {
        if (initial == false) return false;
        
        return ckbd.publicKeyDER(keyDer);
    }
    
    public boolean publicKeyBlob(String keyBlob) {
        if (initial == false) return false;
        
        return ckbd.publicKeyBlob(keyBlob);
    }
    
    public boolean publicKeyAppModulus(String keyModulus) {
        if (initial == false) return false;
        
        return ckbd.publicKeyAppModulus(keyModulus);
    }
    
    public boolean publicKeyAppDER(String keyDer) {
        if (initial == false) return false;
        
        return ckbd.publicKeyAppDER(keyDer);
    }
    
    public boolean publicKeyAppBlob(String keyBlob) {
        if (initial == false) return false;
        
        return ckbd.publicKeyAppBlob(keyBlob);
    }
    
    public boolean publicKeyECC(String x,String y) {
        return ckbd.publicKeyECC(x, y);
    }

    public short verify() {
        if (initial == false) return -1;
        
        return ckbd.verify();
    }
    public void clear() {
        if (initial == false) return;
        
        ckbd.clearPlaintext();
    }
    
    public short getLength() {
        if (initial == false) return 0;
        
        return ckbd.getPlaitextLen();
    }
    
    public long getVersion() {
        if (initial == false) return 0;
        
        return ckbd.getVersion();
    }
    
    public char getComplexDegree() {
        if (initial == false) return 0;
        
        return ckbd.getComplexDegree();
    }
    
    public String lastError() {
        if (initial == false) return null;
        
        return ckbd.lastError();
    }
    
    public void setChallengeCode(byte[] code) {
        if (initial == false) return;
        
        ckbd.setChallengeCode(code);
    }
    
    public byte[] getEncryptedPinCode() {
        if (initial == false) return null;
        
        return ckbd.getEncryptedPinCode();
    }
    
    public void setCalcFactor(String factor) {
        if (initial == false) return;
        
        ckbd.setCalcFactor(factor);
    }
    
    public void setContentType(int contentType) {
        if (initial == false) return;
        
        ckbd.setContentType(contentType);
    }
    
    public void setSoftkbdType(short softkbdType) {
        if (initial == false) return;
        
        ckbd.setKbdType(softkbdType);
    }
    
    public void setSoftkbdStyle(short softkbdStype) {
        if (initial == false) return;
        
        ckbd.setKbdStyle(softkbdStype);
    }
    
    public void setSoftkbdView(short softkbdView) {
        if (initial == false) return;
        
        attrSet.softkbdView = softkbdView;
    }

    
    public void setSoftkbdMode(short softkbdMode) {
        if (initial == false) return;
        
        ckbd.setKbdMode(softkbdMode);
    }
    
    public void setAccepts(String accepts) {
        if (initial == false) return;
        
        ckbd.setAccepts(accepts);
    }
    
    public void setMaxLength(short maxlength) {
        if (initial == false) return;
        
        ckbd.setMaxLength(maxlength);
    }
    
    public void setMinLength(short minlength) {
        if (initial == false) return;
        
        ckbd.setMinLength(minlength);
    }
    
    public void setMode(short mode) {
        if (initial == false) return;
        
        ckbd.setMode(mode);
    }
    
    public void setAlgorithmCode(String algorithmCode) {
        ckbd.setAlgorithmCode(algorithmCode);
    }
    
    public void setHashRandom(String salt) {
        ckbd.setHashRandom(salt);
    }
    
    public void setDictionaryFilter(String dictFilter) {
        ckbd.setDictionaryFilter(dictFilter);
    }
    
    public String getReceiverAction() {
        return CET_RECEIVED + "_" + ckbd.kbd_handle;
    }
    
    public String getMeasureValue() {
        return ckbd.getMeasureValue();
    }
    
    public void setPopMode(boolean mode) {
        popMode = mode;
        ckbd.setPopMode(mode);
    }
    
    public void setSoftkbdRandom(boolean isRandom) {
        ckbd.setKbdRandom(isRandom);
    }
    
    public void setFinishMode(short mode) {
        ckbd.setFinishMode(mode);
    }
    
    public void setMaskChar(char maskChar) {
        attrSet.maskChar = maskChar;
    }
    
    public char getMaskChar() {
        return attrSet.maskChar;
    }
    
    public void setVibrator(boolean flag) {
        attrSet.kbdVibrator = flag;
        ckbd.setKbdVibrator(flag);
    }
    
    public void setSwitchMode(short switchMode) {
        ckbd.setSwitchMode(switchMode);
    }
    
    public void setZoomOut(float zoomOutX, float zoomOutY) {
        attrSet.isZoomOut = true;
        attrSet.zoomOutScaleX = zoomOutX;
        attrSet.zoomOutScaleY = zoomOutY;
    }
    
 }
