package cn.cloudcore.iprotect.plugin;

import android.content.Context;
import android.text.Editable;
import android.text.TextWatcher;
import android.util.AttributeSet;
import android.view.Gravity;
import android.view.View;
import android.view.ViewGroup;
import android.widget.LinearLayout;
import android.widget.RelativeLayout;
import cn.cloudcore.iprotect.view.CircleView;

public class CPayEditText extends RelativeLayout{

    private CEditText cEditText;
    private Context context;

    private LinearLayout linearLayout; //文本密码的文本
    private CircleView[] circleViews;

    private int inputNumber = 6;      


    public CPayEditText(Context context) {
        this(context, null);
    }

    public CPayEditText(Context context, AttributeSet attrs) {
        this(context, attrs, 0);
    }

    public CPayEditText(Context context, AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
        this.context = context;
    }
    
    /**
     * @param attr               密码键盘属性集合
     * @param backgroundDrawable 输入框背景，见示例中的drawable示例
     * @param passLength         密码长度，根据密码长度，生成相应的输入个数
     * @param passColor          圆圈密码颜色
     * @param passCircleRadius   圆圈密码半径大小,单位：dip
     * @param splitLineWidth     分割线宽度，根据backgroundDrawable边框宽度设置,单位：dip
     * @param splitLineColor     分割线颜色，根据backgroundDrawable边框颜色设置
     */
    public boolean initialize(CEditTextAttrSet attr, int backgroundDrawable, 
            int passLength, int passColor, int passCircleRadius,
            float splitLineWidth, int splitLineColor) {
        
        initEdit(attr, backgroundDrawable);
        initStyle(backgroundDrawable, passLength, 
                passColor, passCircleRadius, splitLineWidth, splitLineColor);
        
        return true; 
           
    }

    /**
     * @param backgroundDrawable 输入框背景，见示例中的drawable示例
     * @param passLength         密码长度，根据密码长度，生成相应的输入个数
     * @param passColor          圆圈密码颜色
     * @param passCircleRadius   圆圈密码半径大小,单位：dip
     * @param splitLineWidth     分割线宽度，根据backgroundDrawable边框宽度设置,单位：dip
     * @param splitLineColor     分割线颜色，根据backgroundDrawable边框颜色设置
     */
    private void initStyle(int backgroundDrawable, 
            int passLength, int passColor, int passCircleRadius,
            float splitLineWidth, int splitLineColor) {
        this.inputNumber = passLength;
        //initEdit(backgroundDrawable);
        initShowInput(backgroundDrawable, passColor, passCircleRadius,
                splitLineWidth, splitLineColor);
    }

    /**
     * @param bgcolor
     */
    private void initEdit(CEditTextAttrSet attrSet, int backgroundDrawable) {
        cEditText = new CEditText(context);
        cEditText.initialize(attrSet);
        
        cEditText.setMaxLength((short)inputNumber);
        cEditText.setBackgroundResource(backgroundDrawable);
        
        cEditText.setTextSize(0);
        
        cEditText.addTextChangedListener(new TextWatcher() {
           
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {
            }

            public void onTextChanged(CharSequence s, int start, int before, int count) {}

           
            public void afterTextChanged(Editable s) {
                initDatas(s);
            }
        });
        
        LayoutParams editParams = new LayoutParams(LayoutParams.MATCH_PARENT, 
                ViewGroup.LayoutParams.MATCH_PARENT);
        editParams.addRule(RelativeLayout.CENTER_IN_PARENT, RelativeLayout.TRUE);
        addView(cEditText, editParams);

    }

    
    private void initShowInput(int backgroundDrawable, int passColor, int passCircleRadius, 
            float splitLineWidth, int splitLineColor) {
        
        linearLayout = new LinearLayout(context);
        linearLayout.setBackgroundResource(backgroundDrawable);
        LayoutParams layoutParams = new LayoutParams(
                LayoutParams.MATCH_PARENT,LayoutParams.MATCH_PARENT);
        linearLayout.setLayoutParams(layoutParams);
        linearLayout.setOrientation(LinearLayout.HORIZONTAL);
        addView(linearLayout);
        
        circleViews = new CircleView[inputNumber];
        LinearLayout.LayoutParams circleParams = new LinearLayout.LayoutParams(
                0,LayoutParams.MATCH_PARENT);
        circleParams.weight = 1;
        circleParams.gravity = Gravity.CENTER;

        LinearLayout.LayoutParams splitParams = new LinearLayout.LayoutParams(
                dip2px(context, splitLineWidth),LayoutParams.MATCH_PARENT);
        
        passCircleRadius = dip2px(context, passCircleRadius);

        for(int i = 0; i < circleViews.length; i++) {
            LinearLayout layout = new LinearLayout(context);
            layout.setLayoutParams(layoutParams);
            
            CircleView circleView = new CircleView(context);
            circleView.setCircleRadius(passCircleRadius);
            circleView.setCirCleColor(passColor);
            circleView.setVisibility(View.INVISIBLE);

            circleViews[i] = circleView;
            linearLayout.addView(circleView, circleParams);

            if(i < circleViews.length - 1) {
                View view = new View(context);
                view.setBackgroundColor(splitLineColor);
                linearLayout.addView(view, splitParams);
            }
        }
    }



    /**
     * 根据输入字符，显示密码个数
     * @param s
     */
    public void initDatas(Editable s) {
        
        if (s.length() == 0) {
            for(int i = 0; i < inputNumber; i++) {
                circleViews[i].setVisibility(View.INVISIBLE);
            }
            
            return;
        }
        
        int length = s.length();
        for(int i = 0; i < inputNumber; i++) {
            if(i < length) 
                circleViews[i].setVisibility(View.VISIBLE);
            else
                circleViews[i].setVisibility(View.INVISIBLE);
        }
    }

    public static int dip2px(Context context, float dipValue) {
        final float scale = context.getResources().getDisplayMetrics().density;
        return (int) (dipValue * scale + 0.5f);
    }
    
    public void onDestroy() {
        cEditText.onDestroy();
    }

    public String getValue(String timeStamp) {
        return cEditText.getValue(timeStamp);
    }
    
    public String getPinValue(String timeStamp) {
        return cEditText.getPinValue(timeStamp);
    }
    
    public boolean publicKeyModulus(String keyModulus) {
        return cEditText.publicKeyModulus(keyModulus);
    }
    
    public boolean publicKeyDER(String keyDer) {
        return cEditText.publicKeyDER(keyDer);
    }
    
    public boolean publicKeyBlob(String keyBlob) {
        return cEditText.publicKeyBlob(keyBlob);
    }
    
    public boolean publicKeyAppModulus(String keyModulus) {
        return cEditText.publicKeyAppModulus(keyModulus);
    }
    
    public boolean publicKeyAppDER(String keyDer) {
        return cEditText.publicKeyAppDER(keyDer);
    }
    
    public boolean publicKeyAppBlob(String keyBlob) {
        return cEditText.publicKeyAppBlob(keyBlob);
    }
    
    public boolean publicKeyECC(String x,String y) {
        return cEditText.publicKeyECC(x, y);
    }

    public short verify() {
        return cEditText.verify();
    }
    public void clear() {
        cEditText.clear();
    }
    
    public short getLength() {
        return cEditText.getLength();
    }
    
    public long getVersion() {
        return cEditText.getVersion();
    }
    
    public char getComplexDegree() {
        return cEditText.getComplexDegree();
    }
    
    public String lastError() {
        return cEditText.lastError();
    }
    
    public void openCKbd() {
        cEditText.openCKbd();
    }
    
    public void closeCKbd() {
        cEditText.closeCKbd();
    }
    
    public void setChallengeCode(byte[] code) {
        cEditText.setChallengeCode(code);
    }
    
    public byte[] getEncryptedPinCode() {
        return cEditText.getEncryptedPinCode();
    }
    
    public void setCalcFactor(String factor) {
        cEditText.setCalcFactor(factor);
    }
    
    public void setContentType(int contentType) {
        cEditText.setContentType(contentType);
    }
    
    public void setSoftkbdType(short softkbdType) {
        cEditText.setSoftkbdType(softkbdType);
    }
    
    public void setSoftkbdStype(short softkbdStype) {
        cEditText.setSoftkbdStype(softkbdStype);
    }
    
    public void setSoftkbdMode(short softkbdMode) {
        cEditText.setSoftkbdMode(softkbdMode);
    }
    
    public void setAccepts(String accepts) {
        cEditText.setAccepts(accepts);
    }
    
    public void setMaxLength(short maxlength) {
        cEditText.setMaxLength(maxlength);
    }
    
    public void setMinLength(short minlength) {
        cEditText.setMinLength(minlength);
    }
    
    public void setMode(short mode) {
        cEditText.setMode(mode);
    }
    
    public void setAlgorithmCode(String algorithmCode) {
        cEditText.setAlgorithmCode(algorithmCode);
    }
    
    public void setHashRandom(String salt) {
        cEditText.setHashRandom(salt);
    }
    
    public void setDictionaryFilter(String dictFilter) {
        cEditText.setDictionaryFilter(dictFilter);
    }
    
    public String getReceiverAction() {
        return cEditText.getReceiverAction();
    }
    
    public String getMeasureValue() {
        return cEditText.getMeasureValue();
    }
    
    public void setFinishMode(short finishMode) {
         cEditText.setFinishMode(finishMode);
    }
}
