package cn.cloudcore.iprotect.plugin;

import java.util.ArrayList;
import java.util.List;

import android.content.Context;
import android.util.Log;
import android.webkit.JavascriptInterface;
import android.webkit.WebView;
import cn.cloudcore.iprotect.utils.AlgUtils;

public class CWebProxyBridge {

    private Context context;
    private WebView webView;
    
    private List<CKeyBoard> cKeyBoards;
    private List<String>     saKbdName;    
    
    private final int  KBD_MAX_NUM      = 8;
    
    
    public CWebProxyBridge(Context context, WebView view) {
        this.context = context;
        this.webView = view;
        
        cKeyBoards = new ArrayList<CKeyBoard>();
        saKbdName   = new ArrayList<String>();
    }
    
    @JavascriptInterface
    public boolean createKeyBoard(String keyboardName) {
        
        if (keyboardName == null) return false;
        
        int size = cKeyBoards.size();
        for (int i = 0; i < size; i++) {
        	String name = saKbdName.get(i);
        	if (name.equals(keyboardName)) 
        		return true;
        }

        if(cKeyBoards.size() >= KBD_MAX_NUM) {
            CKeyBoard kbd = cKeyBoards.get(0);
            kbd.destroy();
            
            cKeyBoards.remove(0);
            saKbdName.remove(0);
        }
        
        CKeyBoard newKeyBoard = new CKeyBoard(context, keyboardName);
        CEditTextAttrSet attrSet = new CEditTextAttrSet();
        attrSet.name = keyboardName;
        newKeyBoard.initCSoftKeyBoard(attrSet);
        
        cKeyBoards.add(newKeyBoard);
        saKbdName.add(keyboardName);

        return true;
    }
    
    @JavascriptInterface
    public void setStateInterface(String keyboardName, final String funcName) {
        
        for (int i = 0; i < cKeyBoards.size(); i++) {
            if (!saKbdName.get(i).equals(keyboardName)) continue;
                
            cKeyBoards.get(i).setWebViewStateFuncName(funcName);
            cKeyBoards.get(i).setStateInterface(new CKeyBoardStateInterface() {
                
                public void receive(String funcName, String keyboardName, String state, int length) {
                    webView.loadUrl("javascript:" + funcName + "(\'" + keyboardName +  "\',\'"  + state + "\'," + length + ")");
                }
            });  
                
            return;
        }
    }
    
    @JavascriptInterface
    public void destroyKeyBoard(String keyboardName) {
        for (int i = 0; i < cKeyBoards.size(); i++) {
            if (!saKbdName.get(i).equals(keyboardName)) continue;
              
            cKeyBoards.get(i).destroy();
            cKeyBoards.remove(i);
            return;
        }
    }
    
    @JavascriptInterface
    public void openKeyBoard(String keyboardName) {
        
        for (int i = 0; i < cKeyBoards.size(); i++) {
            if (!saKbdName.get(i).equals(keyboardName)) continue;
            
            cKeyBoards.get(i).openCKbd();
            return;
        }
    }
    
    @JavascriptInterface
    public void closeKeyBoard(String keyboardName) {
        for (int i = 0; i < cKeyBoards.size(); i++) {
            if (!saKbdName.get(i).equals(keyboardName)) continue;
             
            cKeyBoards.get(i).closeCKbd();
            return;
        }
    }
    
    @JavascriptInterface
    public void clearKeyBoard(String keyboardName) {
        for (int i = 0; i < cKeyBoards.size(); i++) {
            if (!saKbdName.get(i).equals(keyboardName)) continue;
            cKeyBoards.get(i).clear();
            return;
        }
    }
    
    @JavascriptInterface
    public void setKeyBoardType(String keyboardName, short kbdType) {
        for (int i = 0; i < cKeyBoards.size(); i++) {
            if (!saKbdName.get(i).equals(keyboardName)) continue;
            
            cKeyBoards.get(i).setSoftkbdType(kbdType);
            if (kbdType == CEditTextAttrSet.SOFT_KBD_QWERTY) {
                cKeyBoards.get(i).setPopMode(false);
            }
            
            return;
        }
    }
    
    @JavascriptInterface
    public void setKeyBoardStyle(String keyboardName, short kbdStyle) {
        for (int i = 0; i < cKeyBoards.size(); i++) {
            if (!saKbdName.get(i).equals(keyboardName)) continue;
            
            cKeyBoards.get(i).setSoftkbdStyle(kbdStyle);
            return;
        }
    }
    
    @JavascriptInterface
    public void setKeyBoardView(String keyboardName, short kbdView) {
        for (int i = 0; i < cKeyBoards.size(); i++) {
            if (!saKbdName.get(i).equals(keyboardName)) continue;
            
            cKeyBoards.get(i).setSoftkbdView(kbdView);
            if (kbdView == CEditTextAttrSet.SOFT_KBD_WITH_EDITTEXT)
                cKeyBoards.get(i).setPopMode(true);
            
            return;
        }
    }
    
    @JavascriptInterface
    public void setKeyBoardRandom(String keyboardName, boolean isRandom) {

        for (int i = 0; i < cKeyBoards.size(); i++) {
            if (!saKbdName.get(i).equals(keyboardName)) continue;
            
            cKeyBoards.get(i).setSoftkbdRandom(isRandom);
            
            return;
        }
    }
    
    @JavascriptInterface
    public void setKeyBoardRandom(String keyboardName, short random) {
        for (int i = 0; i < cKeyBoards.size(); i++) {
            if (!saKbdName.get(i).equals(keyboardName)) continue;
            
            if (random == 0)
                cKeyBoards.get(i).setSoftkbdRandom(false);
            else
                cKeyBoards.get(i).setSoftkbdRandom(true);
            
            return;
        }
    }
    
    @JavascriptInterface
    public String getValue(String keyboardName, String timeStamp) {
        for (int i = 0; i < cKeyBoards.size(); i++) {
            if (!saKbdName.get(i).equals(keyboardName)) continue;
            
            String result = cKeyBoards.get(i).getValue(timeStamp);
            return result;
        }
        
        return null;
    }
    
    @JavascriptInterface
    public String getPinValue(String keyboardName, String timeStamp) {
        for (int i = 0; i < cKeyBoards.size(); i++) {
            if (!saKbdName.get(i).equals(keyboardName)) continue;
            
            String result = cKeyBoards.get(i).getPinValue(timeStamp);
            return result;
        }
        
        return null;
    }
    
    @JavascriptInterface
    public void setMaxLength(String keyboardName, short maxLength) {
        for (int i = 0; i < cKeyBoards.size(); i++) {
            if (!saKbdName.get(i).equals(keyboardName)) continue;
            
            cKeyBoards.get(i).setMaxLength(maxLength);
            return;
        }
        
        return;
    }
    
    @JavascriptInterface
    public void setMinLength(String keyboardName, short minLength) {
        for (int i = 0; i < cKeyBoards.size(); i++) {
            if (!saKbdName.get(i).equals(keyboardName)) continue;
            
            cKeyBoards.get(i).setMinLength(minLength);
            return;
        }
        
        return;
    }
    
    @JavascriptInterface
    public void setSoftkbdMode(String keyboardName, short skbdMode) {
        for (int i = 0; i < cKeyBoards.size(); i++) {
            if (!saKbdName.get(i).equals(keyboardName)) continue;
            
            cKeyBoards.get(i).setSoftkbdMode(skbdMode);
            return;
        }
        
        return;
    }
    
    @JavascriptInterface
    public void setContentType(String keyboardName, int type) {
        for (int i = 0; i < cKeyBoards.size(); i++) {
            if (!saKbdName.get(i).equals(keyboardName)) continue;
            
            cKeyBoards.get(i).setContentType(type);
            return;
        }
        
        return;
    }
    
    @JavascriptInterface
    public boolean publicKeyModulus(String keyboardName, String modulus) {
        for (int i = 0; i < cKeyBoards.size(); i++) {
            if (!saKbdName.get(i).equals(keyboardName)) continue;
            
            return cKeyBoards.get(i).publicKeyModulus(modulus);
        }
        
        return false;
    }
    
    @JavascriptInterface
    public boolean publicKeyAppModulus(String keyboardName, String modulus) {
        for (int i = 0; i < cKeyBoards.size(); i++) {
            if (!saKbdName.get(i).equals(keyboardName)) continue;
            
            return cKeyBoards.get(i).publicKeyAppModulus(modulus);
        }
        
        return false;
    }
    
    @JavascriptInterface
    public boolean publicKeyECC(String keyboardName, String eccX, String eccY) {
        for (int i = 0; i < cKeyBoards.size(); i++) {
            if (!saKbdName.get(i).equals(keyboardName)) continue;
            
            return cKeyBoards.get(i).publicKeyECC(eccX, eccY);
        }
        
        return false;
    }
    
    @JavascriptInterface
    public short verify(String keyboardName) {
        for (int i = 0; i < cKeyBoards.size(); i++) {
            if (!saKbdName.get(i).equals(keyboardName)) continue;
            
            return cKeyBoards.get(i).verify();
        }
        
        return -1;
    }
    
    @JavascriptInterface
    public short getLength(String keyboardName) {
        for (int i = 0; i < cKeyBoards.size(); i++) {
            if (!saKbdName.get(i).equals(keyboardName)) continue;
            
            return cKeyBoards.get(i).getLength();
        }
        
        return 0;
    }
    
    @JavascriptInterface
    public long getVersion(String keyboardName) {
        for (int i = 0; i < cKeyBoards.size(); i++) {
            if (!saKbdName.get(i).equals(keyboardName)) continue;
            
            return cKeyBoards.get(i).getVersion();
        }
        
        return 0;
    }
    
    @JavascriptInterface
    public char getComplexDegree(String keyboardName) {
        for (int i = 0; i < cKeyBoards.size(); i++) {
            if (!saKbdName.get(i).equals(keyboardName)) continue;
            
            return cKeyBoards.get(i).getComplexDegree();
        }
        
        return 'E';
    }
    
    @JavascriptInterface
    public String lastError(String keyboardName) {
        for (int i = 0; i < cKeyBoards.size(); i++) {
            if (!saKbdName.get(i).equals(keyboardName)) continue;
            
            return cKeyBoards.get(i).lastError();
        }
        
        return null;
    }
    
    @JavascriptInterface
    public void setAccepts(String keyboardName, String accepts) {
        for (int i = 0; i < cKeyBoards.size(); i++) {
            if (!saKbdName.get(i).equals(keyboardName)) continue;
            
            cKeyBoards.get(i).setAccepts(accepts);
            return;
        }
    }
    
    @JavascriptInterface
    public void setDictionaryFilter(String keyboardName, String dictFilter) {
        for (int i = 0; i < cKeyBoards.size(); i++) {
            if (!saKbdName.get(i).equals(keyboardName)) continue;
            
            cKeyBoards.get(i).setDictionaryFilter(dictFilter);
            return;
        }
    }
    
    @JavascriptInterface
    public String getMeasureValue(String keyboardName) {
        for (int i = 0; i < cKeyBoards.size(); i++) {
            if (!saKbdName.get(i).equals(keyboardName)) continue;
            
            return cKeyBoards.get(i).getMeasureValue();
        }
        
        return null;
    }
    
    @JavascriptInterface
    public void setFinishMode(String keyboardName, short mode) {
        for (int i = 0; i < cKeyBoards.size(); i++) {
            if (!saKbdName.get(i).equals(keyboardName)) continue;
            
            cKeyBoards.get(i).setFinishMode(mode);
            return;
        }
    }
    
    @JavascriptInterface
    public void setZoomOut(String keyboardName, float zoomOutX, float zoomOutY) {
        for (int i = 0; i < cKeyBoards.size(); i++) {
            if (!saKbdName.get(i).equals(keyboardName)) continue;
            
            cKeyBoards.get(i).setZoomOut(zoomOutX, zoomOutY);
            return;
        }
    }
    
    @JavascriptInterface
    public void setAlgorithmCode(String keyboardName, String algorithmCode) {
        for (int i = 0; i < cKeyBoards.size(); i++) {
            if (!saKbdName.get(i).equals(keyboardName)) continue;
            
            cKeyBoards.get(i).setAlgorithmCode(algorithmCode);
            return;
        }
    }
    
    @JavascriptInterface
    public void setVibrator(String keyboardName, boolean isVibrator) {
        for (int i = 0; i < cKeyBoards.size(); i++) {
            if (!saKbdName.get(i).equals(keyboardName)) continue;
            
            cKeyBoards.get(i).setVibrator(isVibrator);
            return;
        }
    }
    
    @JavascriptInterface
    public int getCipherText(String appPubKey, String encPubKey, String algCode, String timestamp, 
            String jsonStr, StringBuffer ciphertext) {
        return AlgUtils.getCipherText(appPubKey, encPubKey, algCode, timestamp, jsonStr, ciphertext);
    }
}
