package cn.cloudcore.iprotect.service;


import android.app.Service;
import android.content.Intent;
import android.graphics.Color;
import android.graphics.PixelFormat;
import android.os.Binder;
import android.os.Build;
import android.os.Handler;
import android.os.IBinder;
import android.util.DisplayMetrics;
import android.util.Log;
import android.view.Gravity;
import android.view.View;
import android.view.WindowManager;
import android.view.WindowManager.LayoutParams;
import android.widget.FrameLayout;
import android.widget.RelativeLayout;
import cn.cloudcore.iprotect.plugin.CEditTextAttrSet;
import cn.cloudcore.iprotect.plugin.CKbdJniLib;
import cn.cloudcore.iprotect.view.CEditTextView;
import cn.cloudcore.iprotect.view.CKeyBoardHead;
import cn.cloudcore.iprotect.view.CKeyBoardSet;
import cn.cloudcore.iprotect.view.CKeyBoardTopLayer;
import cn.cloudcore.iprotect.view.CKeyBoardView;
import cn.cloudcore.iprotect.view.BottomExToast;

public class CKeyBoardService extends Service {

    private WindowManager windowManager;
    private WindowManager.LayoutParams wmLayoutParams;
        
    private CKeyBoardView  cKeyBoardView;
    private CKeyBoardHead  cKeyBoardHead;
    private CKeyBoardTopLayer cKeyBoardTopLayer;
    
    private RelativeLayout cKeyBoardFun;
    private View gapView;
    private int  gapHeight    = 0; 
    
    private int cFunHeight;
    private int cKeyBoardHeight;
    private int cKeyBoardWidth;
    private int statusBarHeight;
    
    private String cKeyBoardName     = null;
    private CKeyBoardSet cKeyBoardSet;
    
    private CEditTextView cEditTextView;
    private BottomExToast bottomExToast;
    private BottomExToast centerExToast;
    
    private boolean exToastFlag     = false;
    
    public static final int delayTime  = 120;
    private int layoutParamsType    = LayoutParams.TYPE_TOAST;
    //private boolean isXiaomiPhone   = false;
    private boolean isLowVersion    = false;
    private static final String TAG = "CKeyBoardService";

    @Override
    public IBinder onBind(Intent intent) {
        return new CKeyBoardBuild();
    }

    @Override
    public void onCreate() {
        super.onCreate();
        initFloatType();
        createCKeyBoardView();
    }

    @Override
    public void onDestroy() {
        destroyCKeyBoardView();
        super.onDestroy();
    }
    
    private void initFloatType() {
        int versionSDK = Build.VERSION.SDK_INT;
        if (versionSDK < Build.VERSION_CODES.KITKAT) {
            layoutParamsType = LayoutParams.TYPE_PHONE;
            isLowVersion = true;
        }
        
        if ("Xiaomi".equalsIgnoreCase(Build.MANUFACTURER)) {
            exToastFlag = true;
        } else if ("MEIZU".equalsIgnoreCase(Build.MANUFACTURER)) {
            exToastFlag = true;
        } else {
            exToastFlag = false;
        }
        
    }
    
    private void createCKeyBoardView() {
        
        initKeyBoardView();
        initKeyBoardBody();
        initKeyBoardHead();
        
        
        cKeyBoardName = "1111111111";
        CEditTextAttrSet cEditTextAttrSet = new CEditTextAttrSet();
        cEditTextAttrSet.name = cKeyBoardName;
        
        cKeyBoardSet = new CKeyBoardSet(getApplicationContext());
        
        hideCKeyBoardView();
    }
    
    private void destroyCKeyBoardView() {
        if (isLowVersion || !exToastFlag) {
            if (cKeyBoardFun != null) {
                windowManager.removeView(cKeyBoardFun);
                cKeyBoardFun = null;
            }
            
            if (cKeyBoardView != null) {
                windowManager.removeView(cKeyBoardView);
                cKeyBoardView = null;
            }
        } else {
            bottomExToast = null;
            centerExToast = null;
        }
    }
    
    public CKbdJniLib onInitialize(CEditTextAttrSet attrSet) {
//        return cKeyBoardView.onInitialize(attrSet);
    	return null;
    }
    
    private void showCKeyBoardView() {
//        if (gapHeight == 0) {
//            RelativeLayout.LayoutParams params = (RelativeLayout.LayoutParams)
//                    gapView.getLayoutParams();
//            params.height= gapHeight = statusBarHeight / 2;
//            gapView.setLayoutParams(params);
//        }
//
//        if (isLowVersion || !exToastFlag) {
//            cKeyBoardView.setVisibility(View.VISIBLE);
//            cKeyBoardFun.setVisibility(View.GONE);
//        } else {
//            bottomExToast.show();
//            //centerExToast.show();
//        }
    	
    	cKeyBoardSet.show();
    	
    }
    
    public boolean showCKeyBoardView(CEditTextView view, int statusBarHeight, 
            CKeyBoardFinishCallBack finishCallBack) {        
        registerCEditTextView(view, finishCallBack);
        
        this.statusBarHeight = statusBarHeight;
        //showHandler.sendEmptyMessageDelayed(10000, delayTime);
        showCKeyBoardView();     // 原生显示，delay时间由EditText控制

        return true;
    }
    
    public boolean showCKeyBoardView(String cKeyBoardName, int statusBarHeight, 
            CKeyBoardUpdateCallBack updateCallBack, CKeyBoardFinishCallBack finishCallBack) {
        registerCKeyBoardName(cKeyBoardName, updateCallBack, finishCallBack);
        
        this.statusBarHeight = statusBarHeight;
        showHandler.sendEmptyMessageDelayed(10000, delayTime / 2);
        
        return true;
    }

    public String hideCKeyBoardView() {
//        try {
//            if (isLowVersion || !exToastFlag) {
//                cKeyBoardView.setVisibility(View.INVISIBLE);
//                cKeyBoardFun.setVisibility(View.INVISIBLE);
//                
//                unregisterCEditTextView();
//            } else {
//                if (bottomExToast != null) {
//                    bottomExToast.hide();
//                }
//                
//                if (centerExToast != null) {
//                    centerExToast.hide();
//                }
//                
//                unregisterCKeyBoardName();
//            }
//        } catch (Exception e) {}
//        
//        //Log.e("CKeyBoardView", "hideCKeyBoardView");
//        return cKeyBoardName;
    	cKeyBoardSet.dismiss();
    	return cKeyBoardName; 
    }
    
    public boolean isShowCKeyBoardView(CEditTextView view) {
        if (view == null) return false;
        if (cKeyBoardView == null) return false;
        
        String keyboardName = cKeyBoardView.getCKeyBoardName();
        if (keyboardName == null || keyboardName.length() == 0)
            return false;
        
        String edittextName = view.getCEditTextName();
        if (edittextName == null || edittextName.length() == 0)
            return false;
        
        if (!edittextName.equals(keyboardName)) return false;
        
        if (!isLowVersion && exToastFlag) {
            return bottomExToast.isShown();
        } else {
            return cKeyBoardView.isShown();    
        }
    }
    
    private void registerCKeyBoardName(String name, CKeyBoardUpdateCallBack updateCallBack,
            CKeyBoardFinishCallBack finishCallBack) {
        if (name == null) return;
        if (cKeyBoardName != null && cKeyBoardName.equals(name)) {
            return;
        }
            
        cKeyBoardName = name;
//        cKeyBoardView.registerCKeyBoardName(cKeyBoardName, updateCallBack, 
//                finishCallBack);
//        cKeyBoardHead.registerCKeyBoardName(cKeyBoardName);
//        cKeyBoardTopLayer.registerCKeyBoardName(cKeyBoardName);
        
        registerExToast();
    }
    
    private void registerCEditTextView(CEditTextView view, CKeyBoardFinishCallBack 
            finishCallBack) {
        if (cEditTextView == view) return;

        cEditTextView = view;
        cKeyBoardView.registerCEditTextView(cEditTextView, finishCallBack);
//        cKeyBoardHead.registerCEditTextView(cEditTextView);
//        cKeyBoardTopLayer.registerCEditTextView(cEditTextView);
        
        registerExToast();
        //Log.w(TAG, "  registerCEditTextView:" + cEditTextView.getCEditTextName());
    }
    
    private void registerExToast() {
        if (isLowVersion || !exToastFlag) 
            return;

        bottomExToast = new BottomExToast(getApplicationContext());
        bottomExToast.setGravity(Gravity.BOTTOM);
        bottomExToast.setHeight(cKeyBoardHeight);
        bottomExToast.setView(cKeyBoardView);
        
        centerExToast = new BottomExToast(getApplicationContext());
        centerExToast.setDuration(BottomExToast.LENGTH_ALWAYS);
        centerExToast.setGravity(Gravity.CENTER);
        centerExToast.setHeight(cFunHeight);
        centerExToast.setView(cKeyBoardFun);
    }
    
    private void unregisterCEditTextView() {
//        cKeyBoardView.unregisterCEditTextView();
        cEditTextView = null;
        cKeyBoardName = null;
        
        bottomExToast = null;
        centerExToast = null;
    }
    
    private void unregisterCKeyBoardName() {
//        cKeyBoardView.unregisterCEditTextView();
        cEditTextView = null;
        cKeyBoardName = null;
        
        bottomExToast = null;
        centerExToast = null;
    }
    
    public void onFinishInput(CEditTextView view) {
        String cEditTextName = view.getCEditTextName();
        hideCKeyBoardView();
        
        
//        cKeyBoardTopLayer.onFinishInput(cEditTextName);
//        cKeyBoardHead.onFinishInput(cEditTextName);
//        cKeyBoardView.onFinishInput(cEditTextName);
    }
    
    public void onFinishInput(String cKeyBoardName) {
        hideCKeyBoardView();
        
//        cKeyBoardHead.onFinishInput(cKeyBoardName);
//        cKeyBoardTopLayer.onFinishInput(cKeyBoardName);
//        cKeyBoardView.onFinishInput(cKeyBoardName);
    }

    /*
     * 使用内部类，返回当前的service的实例，用于activity调用service的各种方法
     */
    public class CKeyBoardBuild extends Binder {
        public CKeyBoardService getCKeyBoardService() {
            return CKeyBoardService.this;
        }
    }
    
    private void initKeyBoardBody() {
        wmLayoutParams = new WindowManager.LayoutParams();
        
        wmLayoutParams.type    = layoutParamsType;
        wmLayoutParams.format  = PixelFormat.RGB_888;
        wmLayoutParams.flags   = LayoutParams.FLAG_NOT_FOCUSABLE | LayoutParams.FLAG_SECURE;
        //wmLayoutParams.flags   = LayoutParams.FLAG_NOT_FOCUSABLE;
        wmLayoutParams.gravity = Gravity.BOTTOM;
        //wmLayoutParams.windowAnimations = android.R.style.Animation_Dialog;
        
        wmLayoutParams.x       = 0;
        wmLayoutParams.y       = 0;
        wmLayoutParams.width   = cKeyBoardWidth;
        wmLayoutParams.height  = cKeyBoardHeight;
        
        cKeyBoardView = new CKeyBoardView(getApplication()); 
        
        if (isLowVersion || !exToastFlag) {
            windowManager.addView(cKeyBoardView, wmLayoutParams);
        }
    }
    

    private void initKeyBoardHead() {
        
        WindowManager.LayoutParams headLayoutParams = new WindowManager.LayoutParams();
        
        headLayoutParams.type    = LayoutParams.TYPE_TOAST;
        headLayoutParams.format  = PixelFormat.RGB_888;
        headLayoutParams.flags   = LayoutParams.FLAG_NOT_FOCUSABLE | LayoutParams.FLAG_NOT_TOUCHABLE;
        headLayoutParams.gravity = Gravity.CENTER;
        
        headLayoutParams.x       = 0;
        headLayoutParams.y       = 0;
        headLayoutParams.width   = cKeyBoardWidth;
        headLayoutParams.height  = cFunHeight;
        
        cKeyBoardFun = new RelativeLayout(getApplication());
        gapView = new View(getApplication());
        cKeyBoardFun.setBackgroundColor(Color.argb(0, 0, 0, 0));
        
        RelativeLayout.LayoutParams gapParams = new RelativeLayout.LayoutParams(
                RelativeLayout.LayoutParams.MATCH_PARENT, 1);
        gapParams.addRule(RelativeLayout.ALIGN_PARENT_BOTTOM);
        gapView.setId(1);
        
        cKeyBoardFun.addView(gapView, gapParams);
        
        
        cKeyBoardHead = new CKeyBoardHead(getApplication());
        cKeyBoardTopLayer = new CKeyBoardTopLayer(getApplication());
        
         
                
        RelativeLayout.LayoutParams headParams = new RelativeLayout.LayoutParams(
                RelativeLayout.LayoutParams.MATCH_PARENT, 
                cKeyBoardHeight/4);
        headParams.addRule(RelativeLayout.ABOVE, 1);
        
        FrameLayout multiLayout = new FrameLayout(getApplication());
        cKeyBoardFun.addView(multiLayout, headParams);
        
        
        FrameLayout.LayoutParams multiParams = new FrameLayout.LayoutParams(
                FrameLayout.LayoutParams.MATCH_PARENT, 
                FrameLayout.LayoutParams.MATCH_PARENT);
        multiLayout.addView(cKeyBoardHead, multiParams);
        cKeyBoardTopLayer.bringToFront();
        multiLayout.addView(cKeyBoardTopLayer, multiParams);
        cKeyBoardTopLayer.bringToFront();

        if (isLowVersion || !exToastFlag) {
            windowManager.addView(cKeyBoardFun, headLayoutParams);
        }
    }
    
    
    private void initKeyBoardView() {
        windowManager  = (WindowManager)getApplication().getSystemService(
                getApplication().WINDOW_SERVICE);
        
        DisplayMetrics metrics = new DisplayMetrics();
        windowManager.getDefaultDisplay().getMetrics(metrics);
        
        cKeyBoardWidth  = metrics.widthPixels;
        cKeyBoardHeight = Float.valueOf(metrics.heightPixels * CEditTextAttrSet
                .SOFT_KBD_HEIGHT_SCALE / 10).intValue();
        cFunHeight = metrics.heightPixels - 2 * cKeyBoardHeight;
    }
    
    private Handler showHandler = new Handler() {
        @Override
        public void handleMessage(android.os.Message msg) {
            showCKeyBoardView();
        };
    };
}
