package cn.cloudcore.iprotect.utils;

import android.content.Context;

public class AlgUtils {
    
    public static final String SHA1WithRSA = "SHA1WithRSA_BC";
    public static final String MD5WithRSA  = "MD5WithRSA_BC";
    
    public static String getHMAC(String message, String appModulus, String timestamp) {
        return getNativeHMAC(false, message, appModulus, timestamp);
    }
    
    public static String getHMAC(boolean developMode, String message, String appModulus, String timestamp) {
        return getNativeHMAC(developMode, message, appModulus, timestamp);
    }
    
    public static String get3DES(boolean developMode, String message) {
        return getNative3DES(developMode, message);
    }
    
    public static String get3DES(String message) {
        return getNative3DES(false, message);
    }
    
    public static String getSignData(boolean isTestMode, String digest, String message) {
        return getNativeSign(isTestMode, digest, message);
    }

    public static int getCipherText(String appPubKey, String timestamp, String jsonStr, StringBuffer ciphertext) {
        return getNativeCipherText(appPubKey, null, null, timestamp, jsonStr, ciphertext);
    }
    
    /**
     * 
     * @param appPubKey   应用平台公钥， 如果是rsa，使用公钥模数
     * @param encPubKey   加密平台公钥， 如果是rsa，使用公钥模数, 如果是国密，使用"eccX,eccY", 逗号分隔
     * @param timestamp   时间戳
     * @param jsonStr     必须json字符串，不然不能加密
     * @param ciphertext  返回密文
     * @return 
     *          0   - 成功， ciphertext为加密密文
     *          100 - 参数错误
     *          101 - jsonStr 不是json字符串
     *          其他参数： ERROR_TYPE元素说明如下（枚举类型，ERROR_OK为0，以此递增）:
     *                 ERROR_OK : 加密正常
     *                 ERROR_APPLICATION_PLATFORM_MODULUS : 此次加密需要应用平台模数
     *                 ERROR_ENCRYPTION_PLATFORM_MODULUS : 此次加密需要加密平台模数
     *                 ERROR_DUAL_PLATFORMS_MODULUS : 此次加密需要应用平台模数和加密平台模数
     *                 ERROR_TIMESTAMP : 此次加密需要时间戳
     *                 ERROR_TEXT : 原文不存在
     *                 ERROR_CIPHERED_TEXT : 秘文返回指针不存在
     *                 ERROR_ENCRYPTION : 核心加密算法错误
     *                 ERROR_UNKNOWN_BANK : 加密类型错误（通常一个银行对应一个加密类型）
     *                 ERROR_RANDOM : 伪随机数发生器错误
     *                 ERROR_ANSI_X98_PIN_BLOCK : 转换ANSI X9.8 PIN BLOCK失败
     *                 ERROR_ALLOC_MEMORY : 内存分配错误
     *                 ERROR_PKCS1_PADDING : PKCS1 Padding错误
     *                 ERROR_KEYOU_PADDING : 科友 Padding错误
     *                 ERROR_BASE64_ENCODING : Base64编码错误
     *                 ERROR_BASE64_DECODING : Base64解码错误
     *                 ERROR_HEX2BIN : 16进制字符串转字节码错误
     *                 ERROR_BIN2HEX : 字节码转16进制字符串错误
     *                 ERROR_TEXT_FORMAT : 传入的原文格式不正确
     *                 ERROR_SDHS_PADDING : SDHS Padding错误
     *                 ERROR_DECRYPTION : 解密失败
     *                 ERROR_PARAMETER : 参数错误
     *                 ERROR_CONTEXT : 错误的上下文
     *                 ERROR_SM2_PUBLIC_KEY : 错误的SM2公钥
     *                 ERROR_SM2_PRIVATE_KEY : 错误的SM2私钥
     *                 ERROR_VERIFY : 验签失败
     *                 ERROR_TRIAL_PERIOD_EXCEEDED : 试用版过期
     *                 ERROR_NOT_IMPLEMENT : 未实现
     *                 ERROR_PKCS7PADDING : PKCS7 Padding错误
     *                 ERROR_HMAC : HMAC计算出错
     */
    public static int getCipherText(String appPubKey, String encPubKey, String timestamp, 
            String jsonStr, StringBuffer ciphertext) {
        return getNativeCipherText(appPubKey, encPubKey, null, timestamp, jsonStr, ciphertext);
    }
    
    /**
     * 
     * @param appPubKey   应用平台公钥， 如果是rsa，使用公钥模数
     * @param encPubKey   加密平台公钥， 如果是rsa，使用公钥模数, 如果是国密，使用"eccX,eccY", 逗号分隔
     * @param algCode     设置算法，根据算法获得不同算法的密文
     * @param timestamp   时间戳
     * @param jsonStr     必须json字符串，不然不能加密
     * @param ciphertext  返回密文
     * @return 
     *          0   - 成功， ciphertext为加密密文
     *          100 - 参数错误
     *          101 - jsonStr 不是json字符串
     *          其他参数： ERROR_TYPE元素说明如下（枚举类型，ERROR_OK为0，以此递增）:
     *                 ERROR_OK : 加密正常
     *                 ERROR_APPLICATION_PLATFORM_MODULUS : 此次加密需要应用平台模数
     *                 ERROR_ENCRYPTION_PLATFORM_MODULUS : 此次加密需要加密平台模数
     *                 ERROR_DUAL_PLATFORMS_MODULUS : 此次加密需要应用平台模数和加密平台模数
     *                 ERROR_TIMESTAMP : 此次加密需要时间戳
     *                 ERROR_TEXT : 原文不存在
     *                 ERROR_CIPHERED_TEXT : 秘文返回指针不存在
     *                 ERROR_ENCRYPTION : 核心加密算法错误
     *                 ERROR_UNKNOWN_BANK : 加密类型错误（通常一个银行对应一个加密类型）
     *                 ERROR_RANDOM : 伪随机数发生器错误
     *                 ERROR_ANSI_X98_PIN_BLOCK : 转换ANSI X9.8 PIN BLOCK失败
     *                 ERROR_ALLOC_MEMORY : 内存分配错误
     *                 ERROR_PKCS1_PADDING : PKCS1 Padding错误
     *                 ERROR_KEYOU_PADDING : 科友 Padding错误
     *                 ERROR_BASE64_ENCODING : Base64编码错误
     *                 ERROR_BASE64_DECODING : Base64解码错误
     *                 ERROR_HEX2BIN : 16进制字符串转字节码错误
     *                 ERROR_BIN2HEX : 字节码转16进制字符串错误
     *                 ERROR_TEXT_FORMAT : 传入的原文格式不正确
     *                 ERROR_SDHS_PADDING : SDHS Padding错误
     *                 ERROR_DECRYPTION : 解密失败
     *                 ERROR_PARAMETER : 参数错误
     *                 ERROR_CONTEXT : 错误的上下文
     *                 ERROR_SM2_PUBLIC_KEY : 错误的SM2公钥
     *                 ERROR_SM2_PRIVATE_KEY : 错误的SM2私钥
     *                 ERROR_VERIFY : 验签失败
     *                 ERROR_TRIAL_PERIOD_EXCEEDED : 试用版过期
     *                 ERROR_NOT_IMPLEMENT : 未实现
     *                 ERROR_PKCS7PADDING : PKCS7 Padding错误
     *                 ERROR_HMAC : HMAC计算出错
     */
    public static int getCipherText(String appPubKey, String encPubKey, String algCode, String timestamp, 
            String jsonStr, StringBuffer ciphertext) {
        return getNativeCipherText(appPubKey, encPubKey, algCode, timestamp, jsonStr, ciphertext);
    }
    
    /**
     * @param context  上下文关系
     * @param keyInfo  保存的信息
     * @return
     *      0  :  保存成功
     *      1  :  参数错误
     *      2  :  系统调用出错
     *      3  :  内存分配失败
     *      4  :  数据填充异常
     *      5  :  文件打开失败
     *      6  :  文件内容异常
     */
    
    public static int saveKeyInfo(Context context, String keyInfo) {
        return saveNativeKeyInfo(context, CommonUtils.getUniquePsuedoID(), keyInfo);
    }
    
    /**
     * 
     * @param context  上下文关系
     * @param keyInfo  获取的信息内容
     * @return
     *      0  :  读取成功
     *      1  :  参数错误
     *      2  :  系统调用出错
     *      3  :  内存分配失败
     *      4  :  数据填充异常
     *      5  :  文件打开失败
     *      6  :  文件内容异常
     */
    public static int readKeyInfo(Context context, StringBuffer keyInfo) {
        return readNativeKeyInfo(context, CommonUtils.getUniquePsuedoID(), keyInfo);
    }
    
    private native static String getNativeHMAC(boolean developMode, String message, String appModulus, String timestamp);
    private native static String getNative3DES(boolean developMode, String message);
    
    private native static String getNativeSign(boolean isTestMode, String digest, String message);
    private native static int getNativeCipherText(String appPubKey, String encPubKey, String algCode, String timestamp, 
            String jsonStr, StringBuffer ciphertext);
    
    private native static int saveNativeKeyInfo(Context context, String key, String keyInfo);
    private native static int readNativeKeyInfo(Context context, String key, StringBuffer keyInfo);
    
    static {
        System.loadLibrary(CBuildConfig.loadLibraryName);
    }
    
}
