package cn.cloudcore.iprotect.view;

import java.util.UUID;

import android.app.Activity;
import android.content.BroadcastReceiver;
import android.content.ClipData;
import android.content.ClipboardManager;
import android.content.Context;
import android.content.ContextWrapper;
import android.content.Intent;
import android.content.IntentFilter;
import android.graphics.Rect;
import android.os.Handler;
import android.text.InputType;
import android.util.AttributeSet;
//import android.util.Log;
import android.view.ActionMode;
import android.view.Menu;
import android.view.MenuItem;
import android.view.MotionEvent;
import android.view.View;
import android.view.ViewGroup;
import android.view.WindowManager;
import android.view.inputmethod.InputMethodManager;
import android.widget.EditText;
import cn.cloudcore.iprotect.plugin.CEditTextAttrSet;
import cn.cloudcore.iprotect.plugin.CKbdJniLib;
import cn.cloudcore.iprotect.service.CFocusLoseCallBack;
import cn.cloudcore.iprotect.service.CKeyBoardFinishCallBack;
import cn.cloudcore.iprotect.service.CKeyBoardStateCallBack;

public class CEditTextView extends EditText {

    private int statusBarHeight = 0;

    private int screenHeight = 0;
    private int screenWidth = 0;

    private int cEditTextHeight = 0;
    private int cKeyBoardHeight = 0;

    private int cEditTextPush = 0;
    private boolean pushFlag = false;

    private final int HANDLE_PUSH_CEDITTEXT = 1000;
    private final int HANDLE_SHOW_CKEYBOARD = 1001;
    private final int HANDLE_INIT_STATUS_HEIGHT = 1002;
    private final int HANDLE_SHOW_SYSTEM_INPUT = 1003;

    private final int HANDLE_SHOW_CURSOR = 2000;
    private final int HANDLE_HIDE_CURSOR = 2001;

    private static final int delayTime = 150;
    // private static final int delayTime = 60;

    private int delayCKeyBoard;
    private int delayCEditText;
    private int delaySystemInput;

    private View rootView;
    private Activity activity;
    private Context cETContext;

    private CEditTextAttrSet cTextAttrSet;
    private CKeyBoardSet cKeyBoardSet;

    private CKbdJniLib cKbdJniLib;
    private boolean hasInitialize = false;

    private Rect rootViewRect = null;
    private int softInputMode; // pop system keyboard, from backgroup

    private CKeyBoardFinishCallBack finishCallBack = null;
    private CFocusLoseCallBack focusLoseCallBack = null;
    private CKeyBoardStateCallBack stateCallBack = null;
    private final String notInitialize = "uninitialized";

    private ClipData clipData = null;

    private Thread cursorThread;
    private final String TAG = "CEditTextView";

    public CEditTextView(Context context) {
        super(context);
        this.onConstruction(context);
    }

    public CEditTextView(Context context, AttributeSet attrs) {
        super(context, attrs);
        this.onConstruction(context);
    }

    public CEditTextView(Context context, AttributeSet attrs, int defStyle) {
        super(context, attrs, defStyle);
        this.onConstruction(context);
    }

    private void onConstruction(Context context) {
        cETContext = context;
        activity = getActivity(context);
        delayCEditText = delayTime * 3 / 2;
        delayCKeyBoard = delayTime;
        delaySystemInput = delayTime * 3;

        // registerScreenStateReceiver();

        setCustomSelectionActionModeCallback(new ActionModeCallBackInterceptor());
        setLongClickable(false);
        // setTextIsSelectable(false);
        // setImeOptions(EditorInfo.IME_FLAG_NO_EXTRACT_UI);

    }

    public void initialize(CEditTextAttrSet attrSet) {
        initialize(attrSet, null, null);
    }

    public void initialize(CEditTextAttrSet attrSet, CKeyBoardFinishCallBack finishCallBack) {
        initialize(attrSet, finishCallBack, null);
    }

    public void initialize(CEditTextAttrSet attrSet, CKeyBoardFinishCallBack finishCallBack,
            CKeyBoardStateCallBack stateCallBack) {

        if (attrSet.name.equals("CC-iProtect plugin")) {
            attrSet.name = UUID.randomUUID().toString();
        }

        cTextAttrSet = new CEditTextAttrSet(attrSet);

        this.setSingleLine();
        this.setTextColor(cTextAttrSet.textColor);

        if (cTextAttrSet.switchMode == CEditTextAttrSet.SOFT_KBD_SWITCH_BETWEEN_ALPHA_NUMBER) {
            if (cTextAttrSet.softkbdType == CEditTextAttrSet.SOFT_KBD_MONEY) {
                cTextAttrSet.softkbdType = CEditTextAttrSet.SOFT_KBD_NUMBER;
            }
        }

        if (!hasInitialize) {
            cKeyBoardSet = new CKeyBoardSet(cETContext);
            cKbdJniLib = cKeyBoardSet.onInitialize(this, cTextAttrSet, finishCallBack, stateCallBack);
        }

        this.finishCallBack = finishCallBack;
        hasInitialize = true;

        handler.sendEmptyMessageDelayed(HANDLE_INIT_STATUS_HEIGHT, delayTime * 2);
    }

    public void onDestroy() {
        // unregisterScreenStateReceiver();
        hasInitialize = false;

        if (cKeyBoardSet != null && cKeyBoardSet.isShowing()) {
            cKeyBoardSet.dismiss();
        }
    }

    public void updateContent(int plaitextLen) {
        String str = "";

        for (int i = 0; i < plaitextLen; i++) {
            str = str + cTextAttrSet.maskChar;
        }

        this.setText(str);
        this.setSelection(this.length());
    }

    public void onFinishInput() {
        // if (finishCallBack!= null) clearFocus();
        hideCKeyBoardView();
        // clearFocus();

        // Log.e("CEditTextView", "onFinishInput");
        // Log.w("CEditTextView", "onFinishInput");
    }

    private Activity getActivity(Context context) {
        while (!(context instanceof Activity) && context instanceof ContextWrapper) {
            context = ((ContextWrapper) context).getBaseContext();
        }

        if (context instanceof Activity) {
            return (Activity) context;
        }

        return null;
    }

    private void initStatusBarHeight() {
        if (statusBarHeight == 0) {

            if (cETContext instanceof Activity) {
                rootView = ((ViewGroup) ((Activity) cETContext).findViewById(android.R.id.content)).getChildAt(0);
            } else {
                rootView = getRootView();
            }

            rootViewRect = new Rect();
            getRootView().getGlobalVisibleRect(rootViewRect);

            Activity activity = getActivity(cETContext);
            Rect frame = new Rect();
            activity.getWindow().getDecorView().getWindowVisibleDisplayFrame(frame);

            statusBarHeight = frame.top;
            screenHeight = frame.bottom;
            screenWidth = frame.right;
            cKeyBoardHeight = screenHeight / 2;

            Rect viewFrame = new Rect();
            getGlobalVisibleRect(viewFrame);

            int[] location = new int[2];
            getLocationOnScreen(location);

            if (cETContext instanceof Activity) {
                cEditTextHeight = screenHeight - location[1];
            } else {
                cEditTextHeight = screenHeight - location[1] - (viewFrame.bottom - viewFrame.top);
            }

            if (cEditTextHeight < cKeyBoardHeight) {
                cEditTextPush = cKeyBoardHeight - cEditTextHeight;
            }

            if (cKeyBoardSet.isShowing() && !pushFlag) {
                handler.sendEmptyMessageDelayed(HANDLE_PUSH_CEDITTEXT, delayCEditText);
            }
        }
    }

    synchronized private void pushCEditTextView() {
        // initStatusBarHeight();

        if (cEditTextPush != 0 && !pushFlag) {
            pushFlag = true;
            // rootView.layout(0, -cEditTextPush, screenWidth, screenHeight -
            // cEditTextPush);
            rootView.layout(rootViewRect.left, rootViewRect.top - cEditTextPush, rootViewRect.right,
                    rootViewRect.bottom - cEditTextPush);
        }

        // Log.e("CEditTextView", "pushCEditTextView:"+cEditTextPush);
    }

    synchronized public void pullCEditTextView() {
        if (cEditTextPush != 0 && pushFlag) {
            pushFlag = false;
            // rootView.layout(0, 0, screenWidth, screenHeight);
            rootView.layout(rootViewRect.left, rootViewRect.top, rootViewRect.right, rootViewRect.bottom);
        }
    }

    synchronized public void showCKeyBoardView() {
        // initStatusBarHeight();

        if (cEditTextPush != 0 && !pushFlag) {
            pushFlag = true;
            // rootView.layout(0, -cEditTextPush, screenWidth, screenHeight -
            // cEditTextPush);
            rootView.layout(rootViewRect.left, rootViewRect.top - cEditTextPush, rootViewRect.right,
                    rootViewRect.bottom - cEditTextPush);
        }

        cKeyBoardSet.show();
        if (cTextAttrSet.outsideClose) {
            if (cETContext instanceof Activity) {
                dispatchOnTouch((ViewGroup) activity.getWindow().getDecorView());
            } else {
                dispatchOnTouch((ViewGroup) getRootView());
            }
        }

        if (cursorThread == null || !cursorThread.isAlive()) {

            cursorThread = new Thread(new Runnable() {
                public void run() {
                    try {
                        while (true) {
                            Thread.sleep(500);
                            handler.sendEmptyMessage(HANDLE_HIDE_CURSOR);

                            Thread.sleep(500);
                            handler.sendEmptyMessage(HANDLE_SHOW_CURSOR);
                        }
                    } catch (InterruptedException e) {
                        handler.sendEmptyMessage(HANDLE_SHOW_CURSOR);
                    }
                }
            });
            cursorThread.start();
        }

        rootViewRect = new Rect();
        getRootView().getGlobalVisibleRect(rootViewRect);

        // Log.e("CEditTextView", "showCKeyBoardView");
    }

    public void hideCKeyBoardView() {
        // Log.e("CEditTextView", "hideCKeyBoardView");
        handler.removeMessages(HANDLE_PUSH_CEDITTEXT);
        handler.removeMessages(HANDLE_SHOW_CKEYBOARD);

        if (cursorThread != null && cursorThread.isAlive()) {
            cursorThread.interrupt();
        }

        if (cEditTextPush != 0 && pushFlag) {
            pushFlag = false;
            // rootView.layout(0, 0, screenWidth, screenHeight);
            rootView.layout(rootViewRect.left, rootViewRect.top, rootViewRect.right, rootViewRect.bottom);
        }

        cKeyBoardSet.dismiss();
        // clearFocus();

        // ViewGroup viewGroup = ((ViewGroup)((Activity)cETContext).findViewById(
        // android.R.id.content));
        // viewGroup.removeView(cKeyBoardView);
        // cKeyBoardView.setVisibility(View.INVISIBLE);

        // Log.e("CEditTextView", "hideCKeyBoardView");
    }

    private View.OnTouchListener touchListener = new OnTouchListener() {

        public boolean onTouch(View v, MotionEvent event) {

            if (event.getAction() != MotionEvent.ACTION_UP) {
                return false;
            }

            if (cKeyBoardSet.isShowing()) {
                if (v == CEditTextView.this) {
                    return false;
                }

                hideCKeyBoardView();

                if (v instanceof CEditTextView) {
                    // Log.e("CEditTextView", "v instanceof CEditTextView");
                } else if (v instanceof EditText) {
                    if (event.getAction() == MotionEvent.ACTION_UP) {
                        handler.sendEmptyMessageDelayed(HANDLE_SHOW_SYSTEM_INPUT, delayTime * 3);
                    }
                } else {
                    // Log.e("CEditTextView", " touch other ");
                }
            } else {
                if (v == CEditTextView.this) {
                    v.requestFocus();
                    handler.sendEmptyMessageDelayed(HANDLE_PUSH_CEDITTEXT, delayCEditText);
                    handler.sendEmptyMessageDelayed(HANDLE_SHOW_CKEYBOARD, delayCKeyBoard);

                    return true;
                }
            }

            return false;
        }
    };

    private void dispatchOnTouch(ViewGroup viewGroup) {
        if (viewGroup == null) {
            return;
        }

        int childCount = viewGroup.getChildCount();
        for (int i = 0; i < childCount; i++) {
            View view = viewGroup.getChildAt(i);
            view.setOnTouchListener(touchListener);
            if (view instanceof ViewGroup) {
                dispatchOnTouch((ViewGroup) view);
            }
        }
    }

    @Override
    protected void onVisibilityChanged(View changedView, int visibility) {

        if (visibility == View.INVISIBLE && this.hasFocus()) {
            ((Activity) cETContext).getWindow().setSoftInputMode(WindowManager.LayoutParams.SOFT_INPUT_STATE_UNCHANGED);

            hideCKeyBoardView();
        }

        super.onVisibilityChanged(changedView, visibility);
    }

    @Override
    public void onFocusChanged(boolean focused, int direction, Rect previouslyFocusedRect) {
        InputMethodManager imm;

        if (focused == true) {

            if (cKbdJniLib == null || cTextAttrSet == null) {
                clearFocus(); // 进入就聚焦，防止崩溃，不允许
                return;
            }

            int inputType = this.getInputType();
            this.setInputType(InputType.TYPE_NULL);
            this.setInputType(inputType);
            this.setCursorVisible(true);

            Activity activity = getActivity(cETContext);
            softInputMode = activity.getWindow().getAttributes().softInputMode;

            imm = (InputMethodManager) cETContext.getSystemService(Context.INPUT_METHOD_SERVICE);
            imm.hideSoftInputFromWindow(this.getWindowToken(), 0);

            if (cKbdJniLib != null && cTextAttrSet != null) {
                if (cTextAttrSet.clearWhenOpenKbd == true) {
                    cKbdJniLib.clearPlaintext();
                    updateContent(0);
                }

                handler.sendEmptyMessageDelayed(HANDLE_PUSH_CEDITTEXT, delayCEditText);
                handler.sendEmptyMessageDelayed(HANDLE_SHOW_CKEYBOARD, delayCKeyBoard);
                // } else {
                // showHandler.sendEmptyMessageDelayed(HANDLE_CLEAR_FOCUS, 100);
            }

            ClipboardManager clipboardManager = (ClipboardManager) cETContext
                    .getSystemService(getContext().CLIPBOARD_SERVICE);
            clipData = clipboardManager.getPrimaryClip();
            ClipData data = ClipData.newPlainText("", "");
            clipboardManager.setPrimaryClip(data);

            showCKeyBoardView(); // 防止背景白屏

        } else {
            imm = (InputMethodManager) cETContext.getSystemService(Context.INPUT_METHOD_SERVICE);
            imm.hideSoftInputFromWindow(getWindowToken(), InputMethodManager.HIDE_NOT_ALWAYS);

            this.setCursorVisible(false);
            // hideCKeyBoardView();

            // activity.getWindow()
            // .setSoftInputMode(softInputMode |
            // WindowManager.LayoutParams.SOFT_INPUT_IS_FORWARD_NAVIGATION);
            activity.getWindow().setSoftInputMode(softInputMode);
            if (clipData != null) {
                ClipboardManager clipboardManager = (ClipboardManager) cETContext
                        .getSystemService(getContext().CLIPBOARD_SERVICE);
                clipboardManager.setPrimaryClip(clipData);
                clipData = null;
            }

            if (focusLoseCallBack != null) {
                focusLoseCallBack.onFocusLost();
            }
        }
    }

    private void registerScreenStateReceiver() {
        final IntentFilter filter = new IntentFilter();
        filter.addAction(Intent.ACTION_SCREEN_OFF);

        if (hasInitialize) {
            cETContext.unregisterReceiver(screenStateReceiver);
        }
        cETContext.registerReceiver(screenStateReceiver, filter);
    }

    private void unregisterScreenStateReceiver() {
        cETContext.unregisterReceiver(screenStateReceiver);
    }

    private final BroadcastReceiver screenStateReceiver = new BroadcastReceiver() {
        @Override
        public void onReceive(final Context context, final Intent intent) {
            final String action = intent.getAction();
            if (Intent.ACTION_SCREEN_OFF.equals(action)) {
                if (!hasFocus()) {
                    return;
                }

                hideCKeyBoardView();
            }
        }
    };

    private Handler handler = new Handler() {
        @Override
        public void handleMessage(android.os.Message msg) {
            switch (msg.what) {
            case HANDLE_PUSH_CEDITTEXT:
                // Log.w("CEditTextView", "HANDLE_PUSH_CEDITTEXT");
                pushCEditTextView();
                break;

            case HANDLE_SHOW_CKEYBOARD:
                // Log.w("CEditTextView", "HANDLE_SHOW_CKEYBOARD");
                showCKeyBoardView();
                break;

            case HANDLE_INIT_STATUS_HEIGHT:
                // Log.w("CEditTextView", "HANDLE_INIT_STATUS_HEIGHT");
                initStatusBarHeight();
                break;

            case HANDLE_SHOW_SYSTEM_INPUT:
                // Log.w("CEditTextView", "HANDLE_SHOW_SYSTEM_INPUT");
                InputMethodManager imm = (InputMethodManager) getContext()
                        .getSystemService(Context.INPUT_METHOD_SERVICE);
                imm.toggleSoftInput(0, InputMethodManager.SHOW_FORCED);
                break;
            case HANDLE_HIDE_CURSOR:
                setCursorVisible(false);
                break;

            case HANDLE_SHOW_CURSOR:
                setCursorVisible(true);
                break;

            default:
                break;
            }
        }
    };

    public String getCEditTextName() {
        if (cKbdJniLib != null) {
            return cKbdJniLib.kbd_name;
        }

        return null;
    }

    /**
     * This is a replacement method for the base TextView class' method of the same
     * name. This method is used in hidden class android.widget.Editor to determine
     * whether the PASTE/REPLACE popup appears when triggered from the text
     * insertion handle. Returning false forces this window to never appear.
     *
     * @return false
     */
    @SuppressWarnings("unused")
    boolean canPaste() {
        return false;
    }

    /**
     * This is a replacement method for the base TextView class' method of the same
     * name. This method is used in hidden class android.widget.Editor to determine
     * whether the PASTE/REPLACE popup appears when triggered from the text
     * insertion handle. Returning false forces this window to never appear.
     *
     * @return false
     */
    @Override
    public boolean isSuggestionsEnabled() {
        return false;
    }

    @SuppressWarnings("unused")
    boolean canCut() {
        return false;
    }

    @SuppressWarnings("unused")
    boolean canCopy() {
        return false;
    }

    @SuppressWarnings("unused")
    boolean canSelectAllText() {
        return false;
    }

    @SuppressWarnings("unused")
    boolean canSelectText() {
        return false;
    }

    @SuppressWarnings("unused")
    boolean textCanBeSelected() {
        return false;
    }

    @Override
    public boolean onTextContextMenuItem(int id) {
        return false;
    }

    private class ActionModeCallBackInterceptor implements ActionMode.Callback {

        public boolean onCreateActionMode(ActionMode mode, Menu menu) {
            return false;
        }

        public boolean onPrepareActionMode(ActionMode mode, Menu menu) {
            return false;
        }

        public boolean onActionItemClicked(ActionMode mode, MenuItem item) {
            return false;
        }

        public void onDestroyActionMode(ActionMode mode) {
        }

    }

    @SuppressWarnings("unused")
    public String getValue(String timeStamp) throws Exception {
        if (!hasInitialize) {
            throw new Exception(notInitialize);
        }
        if (cKbdJniLib == null) {
            return null;
        }

        String value = cKbdJniLib.getCiphertext(timeStamp);
        if (value == null) {
            throw new Exception("" + lastError());
        }

        return value;
    }

    @SuppressWarnings("unused")
    public String getPinValue(String timeStamp) throws Exception {
        if (!hasInitialize) {
            throw new Exception(notInitialize);
        }
        if (cKbdJniLib == null) {
            return null;
        }

        String value = cKbdJniLib.getPinCiphertext(timeStamp);
        if (value == null) {
            throw new Exception("" + lastError());
        }

        return value;
    }

    @SuppressWarnings("unused")
    public boolean publicKeyModulus(String keyModulus) {
        return cKbdJniLib != null && cKbdJniLib.publicKeyModulus(keyModulus);
    }

    @SuppressWarnings("unused")
    public boolean publicKeyDER(String keyDer) {
        return cKbdJniLib != null && cKbdJniLib.publicKeyDER(keyDer);
    }

    @SuppressWarnings("unused")
    public boolean publicKeyBlob(String keyBlob) {
        return cKbdJniLib != null && cKbdJniLib.publicKeyBlob(keyBlob);
    }

    @SuppressWarnings("unused")
    public boolean publicKeyAppModulus(String keyModulus) {
        return cKbdJniLib != null && cKbdJniLib.publicKeyAppModulus(keyModulus);
    }

    @SuppressWarnings("unused")
    public boolean publicKeyAppDER(String keyDer) {
        return cKbdJniLib != null && cKbdJniLib.publicKeyAppDER(keyDer);
    }

    @SuppressWarnings("unused")
    public boolean publicKeyAppBlob(String keyBlob) {
        return cKbdJniLib != null && cKbdJniLib.publicKeyAppBlob(keyBlob);
    }

    @SuppressWarnings("unused")
    public boolean publicKeyECC(String x, String y) {
        return cKbdJniLib != null && cKbdJniLib.publicKeyECC(x, y);
    }

    @SuppressWarnings("unused")
    public short verify() {
        if (cKbdJniLib == null) {
            return -1;
        }
        return cKbdJniLib.verify();
    }

    @SuppressWarnings("unused")
    public void clear() {
        if (cKbdJniLib == null) {
            return;
        }

        cKbdJniLib.clearPlaintext();
        this.setText("");
        this.setSelection(0);
    }

    @SuppressWarnings("unused")
    public short getLength() {
        if (cKbdJniLib == null) {
            return 0;
        }
        return cKbdJniLib.getPlaitextLen();
    }

    @SuppressWarnings("unused")
    public long getVersion() {
        if (cKbdJniLib == null) {
            return 0;
        }
        return cKbdJniLib.getVersion();
    }

    @SuppressWarnings("unused")
    public char getComplexDegree() {
        if (cKbdJniLib == null) {
            return 'W';
        }
        return cKbdJniLib.getComplexDegree();
    }

    @SuppressWarnings("unused")
    public String lastError() {
        if (cKbdJniLib == null) {
            return null;
        }
        return cKbdJniLib.lastError();
    }

    @SuppressWarnings("unused")
    public void setChallengeCode(byte[] code) {
        if (cKbdJniLib == null) {
            return;
        }
        cKbdJniLib.setChallengeCode(code);
    }

    @SuppressWarnings("unused")
    public byte[] getEncryptedPinCode() {
        if (cKbdJniLib == null) {
            return null;
        }
        return cKbdJniLib.getEncryptedPinCode();
    }

    @SuppressWarnings("unused")
    public void setCalcFactor(String factor) {
        if (cKbdJniLib == null) {
            return;
        }
        cKbdJniLib.setCalcFactor(factor);
    }

    @SuppressWarnings("unused")
    public void setContentType(short contentType) {
        cTextAttrSet.contentType = contentType;
        if (cKbdJniLib == null) {
            return;
        }
        cKbdJniLib.setContentType(contentType);
    }

    @SuppressWarnings("unused")
    public void setSoftkbdType(short softkbdType) {
        cTextAttrSet.softkbdType = softkbdType;
        if (cKbdJniLib == null) {
            return;
        }
        cKbdJniLib.setKbdType(softkbdType);
    }

    @SuppressWarnings("unused")
    public void setSoftkbdStype(short softkbdStype) {
        cTextAttrSet.softkbdStype = softkbdStype;
        if (cKbdJniLib == null) {
            return;
        }
        cKbdJniLib.setKbdStyle(softkbdStype);
    }

    @SuppressWarnings("unused")
    public void setSoftkbdMode(short softkbdMode) {
        cTextAttrSet.softkbdMode = softkbdMode;
        if (cKbdJniLib == null) {
            return;
        }
        cKbdJniLib.setKbdMode(softkbdMode);
    }

    @SuppressWarnings("unused")
    public void setAccepts(String accepts) {
        cTextAttrSet.accepts = accepts;
        if (cKbdJniLib == null) {
            return;
        }
        cKbdJniLib.setAccepts(accepts);
    }

    @SuppressWarnings("unused")
    public void setMaxLength(short maxlength) {
        cTextAttrSet.maxLength = maxlength;
        if (cKbdJniLib == null) {
            return;
        }
        cKbdJniLib.setMaxLength(maxlength);
    }

    @SuppressWarnings("unused")
    public void setMinLength(short minlength) {
        cTextAttrSet.minLength = minlength;
        if (cKbdJniLib == null) {
            return;
        }
        cKbdJniLib.setMinLength(minlength);
    }

    @SuppressWarnings("unused")
    public void setMode(short mode) {
        if (cKbdJniLib == null) {
            return;
        }
        cKbdJniLib.setMode(mode);
    }

    @SuppressWarnings("unused")
    public void setAlgorithmCode(String algorithmCode) {
        if (cKbdJniLib == null) {
            return;
        }
        cKbdJniLib.setAlgorithmCode(algorithmCode);
    }

    @SuppressWarnings("unused")
    public void setHashRandom(String salt) {
        if (cKbdJniLib == null) {
            return;
        }
        cKbdJniLib.setHashRandom(salt);
    }

    @SuppressWarnings("unused")
    public void setDictionaryFilter(String dictFilter) {
        if (cKbdJniLib == null) {
            return;
        }
        cKbdJniLib.setDictionaryFilter(dictFilter);
    }

    @SuppressWarnings("unused")
    public String getMeasureValue() {
        if (cKbdJniLib == null) {
            return null;
        }
        return cKbdJniLib.getMeasureValue();
    }

    @SuppressWarnings("unused")
    public void setFinishMode(short finishMode) {
        cTextAttrSet.finishMode = finishMode;
        if (cKbdJniLib == null) {
            return;
        }
        cKbdJniLib.setFinishMode(finishMode);
    }

    @SuppressWarnings("unused")
    public void setZoomOut(float zoomOutX, float zoomOutY) {
        cTextAttrSet.isZoomOut = true;
        cTextAttrSet.zoomOutScaleX = zoomOutX;
        cTextAttrSet.zoomOutScaleY = zoomOutY;
    }

    public void setKbdRandom(boolean kbdRandom) {
        cTextAttrSet.kbdRandom = kbdRandom;
        if (cKbdJniLib == null) {
            return;
        }
        cKbdJniLib.setKbdRandom(kbdRandom);
    }

    public void setFocusLoseCallBack(CFocusLoseCallBack callBack) {
        focusLoseCallBack = callBack;
    }

    public void setKeyBoardStateCallBack(CKeyBoardStateCallBack stateCallBack) {
        this.stateCallBack = stateCallBack;
    }

}
