package cn.cloudcore.iprotect.view;

import javax.microedition.khronos.egl.EGLConfig;
import javax.microedition.khronos.opengles.GL10;

import android.content.Context;

import android.graphics.PixelFormat;
import android.opengl.GLSurfaceView;
import android.os.Vibrator;
import android.util.AttributeSet;
import android.util.DisplayMetrics;
import android.util.Log;
import android.view.MotionEvent;
import android.view.WindowManager;
import cn.cloudcore.iprotect.plugin.CEditTextAttrSet;
import cn.cloudcore.iprotect.plugin.CKbdJniLib;
import cn.cloudcore.iprotect.service.CKeyBoardFinishCallBack;

public class CKeyBoardHead extends GLSurfaceView {

    public int    defaultHandle             = -1;
    public String defaultName                 = "CloudCoreDefault";
    
    private  CKbdJniLib cKbdJniLib             = null;
    private boolean hasInitialized      = false;
    
    private CEditTextView cEditTextView;
    private CKeyBoardFinishCallBack finishCallBack;
    
    private Context ckbdContext;
    private CKeyBoardSet cKeyBoardSet;
    
    private Vibrator vibrator          = null;
    private short encryptType;
    
    public CKeyBoardHead(Context context) {
        super(context);
        onConstruct(context);
    }
    
    public CKeyBoardHead(Context context, AttributeSet attrs) {
        super(context, attrs);
        onConstruct(context);
    }
    
    private void onConstruct(Context context) {
        ckbdContext = context;
        
        setZOrderOnTop(true);
        setEGLConfigChooser(8, 8, 8, 8, 16, 0);
        getHolder().setFormat(PixelFormat.TRANSLUCENT);
        
        cKbdJniLib    = new CKbdJniLib(defaultName);
        defaultHandle = cKbdJniLib.kbd_handle;

        CKeyBoardRenderer cKeyBoardRenderer = new CKeyBoardRenderer();
        setRenderer(cKeyBoardRenderer);
    }
    
    public void onInitialize(CEditTextAttrSet attrSet) {
        CKbdJniLib jniLib = new CKbdJniLib(attrSet.name);
        
        cKbdJniLib = jniLib;
        encryptType = cKbdJniLib.getEncryptType();
    }
    
    public void onInitialize(CEditTextView view, CEditTextAttrSet attrSet, CKeyBoardFinishCallBack 
            finishCallBack) {
        
        onInitialize(attrSet);
        
        cEditTextView = view;
        this.finishCallBack = finishCallBack;
    }
    
    private float getFinishPosition() {
        WindowManager windowManager  = (WindowManager)ckbdContext.getSystemService(
                ckbdContext.WINDOW_SERVICE);

        DisplayMetrics metrics = new DisplayMetrics();
        windowManager.getDefaultDisplay().getMetrics(metrics);
        
        //Log.e("onTouchEvent", "widthPixels - " + metrics.widthPixels);

        return metrics.widthPixels * 8.5f / 10;
    }
    
    public void registerCKeyBoardName(CKeyBoardSet cKeyBoardSet,
            CKeyBoardFinishCallBack finishCallBack) {
        this.cKeyBoardSet = cKeyBoardSet;
        this.finishCallBack = finishCallBack;
        
        boolean needVibrator = cKbdJniLib.getKbdVibrator();
        //Log.e("CKeyBoardHead", "needVibrator:" + needVibrator);
        if (needVibrator == true)
            vibrator = (Vibrator) ckbdContext.getSystemService(Context.VIBRATOR_SERVICE);
        else 
            vibrator = null;
    }
    
    @Override
    public boolean onTouchEvent(MotionEvent event) {
        
        float finishPos = getFinishPosition();
        
        //Log.e("onTouchEvent", "finishPos - " + finishPos);
        //Log.e("onTouchEvent", "encryptType - " + encryptType);
        
        if (encryptType != 120) // KSRCB
            return super.onTouchEvent(event); 
        
        if (event.getX() < finishPos) {
            return super.onTouchEvent(event);
        }
        
        if (event.getAction() ==  MotionEvent.ACTION_DOWN) {
            if (vibrator != null) { 
                vibrator.vibrate(40);
            }
            return true;
        }
        
        if (event.getAction() != MotionEvent.ACTION_UP) {
            return super.onTouchEvent(event);
        }
        
        if (cEditTextView != null) { // native use
            if (finishCallBack != null) { // register finish callback
                if (finishCallBack.onFinish()) {
                    cEditTextView.onFinishInput();
                }
            } else {
                cEditTextView.onFinishInput();
            }
        } else {  // html5 use
            if (finishCallBack != null) {
                boolean bRetVal = finishCallBack.onFinish();
                if (bRetVal == true) cKeyBoardSet.dismiss();
            } else {
                cKeyBoardSet.dismiss();
            }
        }

        finishCallBack = null;
     
        return true;
    }
    
    class CKeyBoardRenderer implements GLSurfaceView.Renderer {

        public void onSurfaceCreated(GL10 gl, EGLConfig config) {
            cKbdJniLib.setPopSurfaceCreated();
            hasInitialized = true;
        }

        public void onSurfaceChanged(GL10 gl, int width, int height) {
            cKbdJniLib.setPopSurfaceChanged(width, height);
        }

        public void onDrawFrame(GL10 gl) {
            if (!hasInitialized) {
                cKbdJniLib.setPopSurfaceCreated();
                hasInitialized = true;
            }
            cKbdJniLib.setPopDrawFrame(false);
        }
    }
}
