package cn.cloudcore.iprotect.view;

import android.app.Activity;
import android.app.Dialog;
import android.content.Context;
import android.content.ContextWrapper;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.support.annotation.StyleRes;
import android.util.DisplayMetrics;
//import android.util.Log;
import android.view.KeyEvent;
import android.view.MotionEvent;
import android.view.View;
import android.view.Window;
import android.view.WindowManager;
import android.view.animation.Animation;
import android.view.animation.TranslateAnimation;
import android.widget.FrameLayout;
import android.widget.LinearLayout;
import android.widget.RelativeLayout;
import cn.cloudcore.iprotect.plugin.CEditTextAttrSet;
import cn.cloudcore.iprotect.plugin.CKbdJniLib;
import cn.cloudcore.iprotect.service.CKeyBoardFinishCallBack;
import cn.cloudcore.iprotect.service.CKeyBoardStateCallBack;
import cn.cloudcore.iprotect.service.CKeyBoardUpdateCallBack;

public class CKeyBoardSet extends Dialog {

    private Context context;

    private Animation showAnimation;
    private Animation hideAnimation;

    private RelativeLayout rootLayout;
    private LinearLayout realLayout;
    private FrameLayout auxiLayout;

    private int cKeyBoardWidth;
    private int cKeyBoardBaseHeight;
    private int cKeyBoardAuxiHeight;
    private int cKeyBoardRegion;
    private boolean outsideClose;

    private CEditTextView cEditTextView;
    private CKeyBoardView cKeyBoardView;
    private CKeyBoardHead cKeyBoardHead;
    private CKeyBoardTopLayer cKeyBoardTopLayer;

    private CKeyBoardStateCallBack stateCallBack = null;

    private CKbdJniLib cKbdJniLib;
    private boolean isCordova = false;

    public CKeyBoardSet(@NonNull Context context) {
        super(context, android.R.style.Theme_Translucent);
        this.context = context;

        onConstruction();
    }

    public CKeyBoardSet(@NonNull Context context, @StyleRes int themeResId) {
        super(context, android.R.style.Theme_Translucent);
        this.context = context;

        onConstruction();
    }

    public CKeyBoardSet(@NonNull Context context, boolean cancelable, @Nullable OnCancelListener cancelListener) {
        super(context, android.R.style.Theme_Translucent);
        this.context = context;

        onConstruction();
    }

    private void onConstruction() {
        showAnimation = new TranslateAnimation(Animation.RELATIVE_TO_SELF, 0.0f, Animation.RELATIVE_TO_SELF, 0.0f,
                Animation.RELATIVE_TO_SELF, 1.0f, Animation.RELATIVE_TO_SELF, 0.0f);

        hideAnimation = new TranslateAnimation(Animation.RELATIVE_TO_SELF, 0.0f, Animation.RELATIVE_TO_SELF, 0.0f,
                Animation.RELATIVE_TO_SELF, 0.0f, Animation.RELATIVE_TO_SELF, 1.0f);

        hideAnimation.setAnimationListener(new Animation.AnimationListener() {
            public void onAnimationStart(Animation animation) {
            }

            public void onAnimationRepeat(Animation animation) {
            }

            public void onAnimationEnd(Animation animation) {
                CKeyBoardSet.super.dismiss();
            }
        });

        // 没有效果
        showAnimation.setDuration(100);
        hideAnimation.setDuration(100);

        rootLayout = new RelativeLayout(context);
        realLayout = new LinearLayout(context);
        auxiLayout = new FrameLayout(context);

        cKeyBoardView = new CKeyBoardView(context);
        cKeyBoardHead = new CKeyBoardHead(context);
        cKeyBoardTopLayer = new CKeyBoardTopLayer(context);

        cKeyBoardView.setSecure(true);
        cKeyBoardHead.setSecure(true);
        cKeyBoardTopLayer.setSecure(true);
    }

    @SuppressWarnings("unused")
    public CKbdJniLib onInitialize(CEditTextView cEditTextView, CEditTextAttrSet attrSet,
            CKeyBoardFinishCallBack finishCallBack, CKeyBoardStateCallBack stateCallBack) {
        cKbdJniLib = new CKbdJniLib(attrSet.name);
        cKbdJniLib.setKbdType(attrSet.softkbdType);
        cKbdJniLib.setKbdStyle(attrSet.softkbdStype);
        cKbdJniLib.setAccepts(attrSet.accepts);
        cKbdJniLib.setMaxLength(attrSet.maxLength);
        cKbdJniLib.setMinLength(attrSet.minLength);
        cKbdJniLib.setKbdRandom(attrSet.kbdRandom);
        cKbdJniLib.setContentType(attrSet.contentType);
        cKbdJniLib.setKbdMode(attrSet.softkbdMode);

        if (attrSet.softkbdMode == CEditTextAttrSet.SOFT_KBD_MODE_KDOWN_NOPOP) {
            cKbdJniLib.setPopMode(false);
            cKbdJniLib.setKbdMode(CEditTextAttrSet.SOFT_KBD_MODE_KDOWN_CHANGE);
        } else {
            cKbdJniLib.setKbdMode(attrSet.softkbdMode);
        }
        // jniLib.setKbdMode(attrSet.softkbdMode);
        cKbdJniLib.setSwitchMode(attrSet.switchMode);
        cKbdJniLib.setKbdVibrator(attrSet.kbdVibrator);

        this.outsideClose = attrSet.outsideClose;
        this.cEditTextView = cEditTextView;
        // this.finishCallBack = finishCallBack;
        this.stateCallBack = stateCallBack;

        cKeyBoardView.onInitialize(cEditTextView, attrSet, finishCallBack);
        cKeyBoardHead.onInitialize(cEditTextView, attrSet, finishCallBack);
        cKeyBoardTopLayer.onInitialize(attrSet);

        if (cEditTextView == null) {
            isCordova = true;
        }

        return cKbdJniLib;
    }

    @SuppressWarnings("unused")
    public CKbdJniLib onInitialize(CEditTextAttrSet attrSet, CKeyBoardFinishCallBack finishCallBack) {
        return onInitialize(null, attrSet, finishCallBack, null);
    }

    @SuppressWarnings("unused")
    public CKbdJniLib onInitialize(CEditTextView cEditTextView, CEditTextAttrSet attrSet) {
        return onInitialize(cEditTextView, attrSet, null, null);
    }

    @SuppressWarnings("unused")
    public CKbdJniLib onInitialize(CEditTextAttrSet attrSet) {
        return onInitialize(null, attrSet, null, null);
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        requestWindowFeature(Window.FEATURE_NO_TITLE);
        getWindow().addFlags(WindowManager.LayoutParams.FLAG_SECURE);

        initCKeyBoardLayout();
        setContentView(rootLayout);
        setCanceledOnTouchOutside(false);
    }

    private void initCKeyBoardLayout() {
        initKeyBoardParams();
        initCKeyBoardRootLayout();
        initCKeyBoardAuxiLayout();
        initCKeyBoardBaseLayout();
    }

    private void initKeyBoardParams1() {

        if (cKbdJniLib.getEncryptType() == 120) {
            CEditTextAttrSet.SOFT_KBD_HEIGHT_SCALE = 3.3f;
        }

        if (cKbdJniLib.getHeightType() == 0) {
            CEditTextAttrSet.SOFT_KBD_HEIGHT_SCALE = 4.0f;
        } else if (cKbdJniLib.getHeightType() == 1) {
            CEditTextAttrSet.SOFT_KBD_HEIGHT_SCALE = 3.3f;
        } else if (cKbdJniLib.getHeightType() == 2) {
            CEditTextAttrSet.SOFT_KBD_HEIGHT_SCALE = 3.5f;
        } else {
            CEditTextAttrSet.SOFT_KBD_HEIGHT_SCALE = 4.0f;
        }

        WindowManager windowManager = (WindowManager) context.getSystemService(context.WINDOW_SERVICE);

        DisplayMetrics metrics = new DisplayMetrics();
        windowManager.getDefaultDisplay().getMetrics(metrics);

        // cKeyBoardWidth = metrics.widthPixels;
        cKeyBoardBaseHeight = Float.valueOf(metrics.heightPixels * CEditTextAttrSet.SOFT_KBD_HEIGHT_SCALE / 10)
                .intValue();
        cKeyBoardAuxiHeight = cKeyBoardBaseHeight / 4;

        if (cKbdJniLib.getEncryptType() == 120) {
            cKeyBoardRegion = metrics.heightPixels - cKeyBoardBaseHeight - cKeyBoardAuxiHeight;
        } else {
            cKeyBoardRegion = metrics.heightPixels - cKeyBoardBaseHeight;
        }
    }

    private void initKeyBoardParams() {

        if (cKbdJniLib.getHeightType() == 1) {
            CEditTextAttrSet.SOFT_WIDTH_HEIGHT_SCALE = 0.6818f;
        } else if (cKbdJniLib.getHeightType() == 2) {
            CEditTextAttrSet.SOFT_WIDTH_HEIGHT_SCALE = 0.6429f;
        } else {
            CEditTextAttrSet.SOFT_WIDTH_HEIGHT_SCALE = 0.5625f;
        }

        // Log.e("initKeyBoardParams", "SOFT_WIDTH_HEIGHT_SCALE:"+
        // CEditTextAttrSet.SOFT_WIDTH_HEIGHT_SCALE);

        WindowManager windowManager = (WindowManager) context.getSystemService(context.WINDOW_SERVICE);
        DisplayMetrics metrics = new DisplayMetrics();
        windowManager.getDefaultDisplay().getMetrics(metrics);

        cKeyBoardWidth = metrics.widthPixels;
        cKeyBoardBaseHeight = (int) (cKeyBoardWidth * 4 / 10 / CEditTextAttrSet.SOFT_WIDTH_HEIGHT_SCALE);
        cKeyBoardAuxiHeight = cKeyBoardBaseHeight / 4;

        if (cKbdJniLib.getEncryptType() == 120) {
            cKeyBoardRegion = metrics.heightPixels - cKeyBoardBaseHeight - cKeyBoardAuxiHeight;
        } else {
            cKeyBoardRegion = metrics.heightPixels - cKeyBoardBaseHeight;
        }
    }

    private void initCKeyBoardRootLayout() {
        RelativeLayout.LayoutParams rootParams = new RelativeLayout.LayoutParams(
                RelativeLayout.LayoutParams.MATCH_PARENT, RelativeLayout.LayoutParams.MATCH_PARENT);

        rootParams.addRule(RelativeLayout.ALIGN_PARENT_BOTTOM);

        rootLayout.setLayoutParams(rootParams);
        rootLayout.setVisibility(View.INVISIBLE);

        LinearLayout.LayoutParams subParams = new LinearLayout.LayoutParams(LinearLayout.LayoutParams.MATCH_PARENT,
                LinearLayout.LayoutParams.WRAP_CONTENT);

        realLayout.setLayoutParams(subParams);
        realLayout.setOrientation(LinearLayout.VERTICAL);

        RelativeLayout.LayoutParams realParams = new RelativeLayout.LayoutParams(
                RelativeLayout.LayoutParams.MATCH_PARENT, RelativeLayout.LayoutParams.WRAP_CONTENT);

        realParams.addRule(RelativeLayout.ALIGN_PARENT_BOTTOM);
        rootLayout.addView(realLayout, realParams);
    }

    private void initCKeyBoardAuxiLayout() {
        LinearLayout.LayoutParams auxiParams = new LinearLayout.LayoutParams(LinearLayout.LayoutParams.MATCH_PARENT,
                cKeyBoardAuxiHeight);

        FrameLayout.LayoutParams auxiSubParams = new FrameLayout.LayoutParams(FrameLayout.LayoutParams.MATCH_PARENT,
                FrameLayout.LayoutParams.MATCH_PARENT);

        realLayout.addView(auxiLayout, auxiParams);

        auxiLayout.addView(cKeyBoardHead, auxiSubParams);
        cKeyBoardTopLayer.bringToFront();
        auxiLayout.addView(cKeyBoardTopLayer, auxiSubParams);
        cKeyBoardTopLayer.bringToFront();
    }

    private void initCKeyBoardBaseLayout() {
        LinearLayout.LayoutParams baseParams = new LinearLayout.LayoutParams(LinearLayout.LayoutParams.MATCH_PARENT,
                cKeyBoardBaseHeight);

        realLayout.addView(cKeyBoardView, baseParams);
    }

    @SuppressWarnings("unused")
    public String getCKeyBoardName() {
        if (cKbdJniLib != null) {
            return cKbdJniLib.kbd_name;
        }

        return null;
    }

    @Override
    public boolean onTouchEvent(@NonNull MotionEvent event) {

        if (!outsideClose) {
            return super.onTouchEvent(event);
        }

        if (event.getY() > cKeyBoardRegion) {
            return super.onTouchEvent(event);
        } else {
            if (isCordova && outsideClose) {
                dismiss();
            }

            // return ((Activity) context).dispatchTouchEvent(event);
            if (context instanceof Activity) {
                return ((Activity) context).dispatchTouchEvent(event);
            } else {
                dismiss();
                return false;
            }
        }
    }

    private Activity getActivity(Context context) {
        int count = 1;
        while (!(context instanceof Activity) && context instanceof ContextWrapper) {
            context = ((ContextWrapper) context).getBaseContext();
        }

        if (context instanceof Activity) {
            return (Activity) context;
        }

        return null;
    }

    @Override
    public boolean onKeyDown(int keyCode, KeyEvent event) {
        if (keyCode == KeyEvent.KEYCODE_BACK) {

            if (cEditTextView != null) {
                cEditTextView.hideCKeyBoardView();
                return false;
            }
        }

        return super.onKeyDown(keyCode, event);
    }

    @Override
    public void show() {
        if (isShowing()) {
            return;
        }

        super.show();
        rootLayout.startAnimation(showAnimation);
        rootLayout.setVisibility(View.VISIBLE);

        if (stateCallBack != null) {
            stateCallBack.onStateChanged(1);
        }
    }

    @Override
    public void dismiss() {

        if (!isShowing()) {
            return;
        }
        // super.dismiss();
        rootLayout.startAnimation(hideAnimation);
        if (cEditTextView != null) {
            cEditTextView.pullCEditTextView();
        }

        if (stateCallBack != null) {
            stateCallBack.onStateChanged(0);
        }

        if (cKbdJniLib != null) {
            cKbdJniLib.setSurfaceClose();
        }
    }

    public void showCKeyBoardSet(CKeyBoardUpdateCallBack updateCallBack, CKeyBoardFinishCallBack finishCallBack) {

        cKeyBoardView.registerCKeyBoardName(this, updateCallBack, finishCallBack);
        cKeyBoardHead.registerCKeyBoardName(this, finishCallBack);

        show();
    }
}
