package cn.cloudcore.iprotect.view;

import javax.microedition.khronos.egl.EGLConfig;
import javax.microedition.khronos.opengles.GL10;

import android.content.Context;
import android.opengl.GLSurfaceView;
import android.os.Vibrator;
import android.util.AttributeSet;
import android.util.Log;
import android.view.MotionEvent;
import cn.cloudcore.iprotect.plugin.CEditTextAttrSet;
import cn.cloudcore.iprotect.plugin.CKbdJniLib;
import cn.cloudcore.iprotect.service.CKeyBoardFinishCallBack;
import cn.cloudcore.iprotect.service.CKeyBoardUpdateCallBack;

public class CKeyBoardView extends GLSurfaceView {

    public int defaultHandle = -1;
    public String defaultName = "CloudCoreDefault";

    private Context ckbdContext;
    private CKeyBoardSet cKeyBoardSet;

    private String cKeyBoardName;
    private CEditTextView cEditTextView;
    private CKbdJniLib cKbdJniLib;

    private CKeyBoardUpdateCallBack updateCallBack;
    private CKeyBoardFinishCallBack finishCallBack;

    private Vibrator vibrator = null;
    private short encryptType = 0;

    private final String TAG = "CKeyBoardView";

    public CKeyBoardView(Context context) {
        super(context);
        onConstruct(context);
    }

    public CKeyBoardView(Context context, AttributeSet attrs) {
        super(context, attrs);
        onConstruct(context);
    }

    public void onConstruct(Context context) {
        ckbdContext = context;

        cKbdJniLib = new CKbdJniLib(defaultName);
        defaultHandle = cKbdJniLib.kbd_handle;

        CKeyBoardRenderer cKeyBoardRenderer = new CKeyBoardRenderer();
        setRenderer(cKeyBoardRenderer);
    }

    public void onInitialize(CEditTextAttrSet attrSet) {
        CKbdJniLib jniLib = new CKbdJniLib(attrSet.name);
        Log.e("VERSION", "--- getVersion" + cKbdJniLib.getVersion());

        cKbdJniLib = jniLib;
        cKeyBoardName = jniLib.kbd_name;

        encryptType = cKbdJniLib.getEncryptType();
    }

    public void onInitialize(CEditTextView view, CEditTextAttrSet attrSet, CKeyBoardFinishCallBack finishCallBack) {

        onInitialize(attrSet);

        cEditTextView = view;
        this.finishCallBack = finishCallBack;
    }

    public void registerCKeyBoardName(CKeyBoardSet cKeyBoardSet, CKeyBoardUpdateCallBack updateCallBack,
            CKeyBoardFinishCallBack finishCallBack) {

        this.updateCallBack = updateCallBack;
        this.finishCallBack = finishCallBack;
        this.cKeyBoardSet = cKeyBoardSet;

        boolean needVibrator = cKbdJniLib.getKbdVibrator();
        // Log.e("CKeyBoardView", "needVibrator:" + needVibrator);
        if (needVibrator == true) {
            vibrator = (Vibrator) ckbdContext.getSystemService(Context.VIBRATOR_SERVICE);
        } else {
            vibrator = null;
        }
    }

    public void registerCEditTextView(CEditTextView view, CKeyBoardFinishCallBack finishCallBack) {
        if (cEditTextView == view) {
            return;
        }

        cEditTextView = view;
        this.finishCallBack = finishCallBack;

        boolean needVibrator = cKbdJniLib.getKbdVibrator();
        if (needVibrator == true) {
            vibrator = (Vibrator) ckbdContext.getSystemService(Context.VIBRATOR_SERVICE);
        } else {
            vibrator = null;
        }
    }

    @Override
    public boolean onTouchEvent(MotionEvent event) {

        int retval = 0;

        if (event.getAction() == MotionEvent.ACTION_DOWN || event.getAction() == MotionEvent.ACTION_UP
                || event.getAction() == MotionEvent.ACTION_MOVE) {
            retval = cKbdJniLib.touchEvent(event.getAction(), event.getX(), event.getY());
        } else {
            return super.onTouchEvent(event);
        }

        if (event.getAction() == MotionEvent.ACTION_DOWN) {
            if (vibrator != null) {
                vibrator.vibrate(40);
            }
        }

        if (event.getAction() != MotionEvent.ACTION_UP) {
            return true;
        }

        if (retval == -1) {
            return true;
        } else if (!(retval == 1 || retval == 2)) {
            updateContent();
            return true;
        }

        if (encryptType == 120) {
            return true;
        }

        if (cEditTextView != null) { // native use
            if (finishCallBack != null) { // register finish callback
                if (finishCallBack.onFinish()) {
                    cEditTextView.onFinishInput();
                }
            } else {
                cEditTextView.onFinishInput();
            }
        } else { // html5 use
            if (finishCallBack != null) {
                boolean bRetVal = finishCallBack.onFinish();
                if (bRetVal == true) {
                    cKeyBoardSet.dismiss();
                }

            }
        }

        // finishCallBack = null;
        // updateCallBack = null;
        updateContent();

        return true;
    }

    private void updateContent() {
        if (cEditTextView != null) {
            cEditTextView.updateContent(getPlaitextLen());
            return;
        }

        if (updateCallBack != null) {
            updateCallBack.onUpdate(cKeyBoardName, getPlaitextLen(), getComplexDegree());
        }
    }

    class CKeyBoardRenderer implements GLSurfaceView.Renderer {

        public void onSurfaceCreated(GL10 gl, EGLConfig config) {
            cKbdJniLib.setSurfaceCreated();
        }

        public void onSurfaceChanged(GL10 gl, int width, int height) {
            cKbdJniLib.setSurfaceChanged(width, height, 1);
        }

        public void onDrawFrame(GL10 gl) {
            if (!cKbdJniLib.isInitialized) {
                cKbdJniLib.setSurfaceCreated();
            }
            cKbdJniLib.setDrawFrame();
        }
    }

    private int getPlaitextLen() {
        return cKbdJniLib.getPlaitextLen();
    }

    public char getComplexDegree() {
        return cKbdJniLib.getComplexDegree();
    }

    public String getCKeyBoardName() {
        return cKbdJniLib.kbd_name;
    }

}
