package cn.cloudcore.iprotect.view;

import android.content.Context;
import android.text.Editable;
import android.text.TextWatcher;
import android.util.AttributeSet;
import android.view.Gravity;
import android.view.View;
import android.view.ViewGroup;
import android.widget.LinearLayout;
import android.widget.RelativeLayout;
import cn.cloudcore.iprotect.plugin.CEditTextAttrSet;
import cn.cloudcore.iprotect.service.CKeyBoardFinishCallBack;

public class CPayEditTextView extends RelativeLayout {

    private CEditTextView cEditTextView;
    private Context context;

    private LinearLayout linearLayout; // 文本密码的文本
    private CircleView[] circleViews;

    private int inputNumber = 6;

    public CPayEditTextView(Context context) {
        this(context, null);
    }

    public CPayEditTextView(Context context, AttributeSet attrs) {
        this(context, attrs, 0);
    }

    public CPayEditTextView(Context context, AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
        this.context = context;
    }

    /**
     * @param attr
     *            密码键盘属性集合
     * @param finishCallBack
     *            完成回调，在关闭键盘时，如果需要处理一些事物，可以在这类处理，返回false表示不能关闭
     * @param backgroundDrawable
     *            输入框背景，见示例中的drawable示例
     * @param passLength
     *            密码长度，根据密码长度，生成相应的输入个数
     * @param passColor
     *            圆圈密码颜色
     * @param passCircleRadius
     *            圆圈密码半径大小,单位：dip
     * @param splitLineWidth
     *            分割线宽度，根据backgroundDrawable边框宽度设置,单位：dip
     * @param splitLineColor
     *            分割线颜色，根据backgroundDrawable边框颜色设置
     */
    public boolean initialize(CEditTextAttrSet attr, CKeyBoardFinishCallBack finishCallBack, int backgroundDrawable,
            int passLength, int passColor, int passCircleRadius, float splitLineWidth, int splitLineColor) {
        if (attr.maxLength != passLength) {
            attr.maxLength = (short) passLength;
        }

        inputNumber = passLength;
        initEdit(attr, finishCallBack, backgroundDrawable);
        initStyle(backgroundDrawable, passLength, passColor, passCircleRadius, splitLineWidth, splitLineColor);

        return true;

    }

    /**
     * @param backgroundDrawable
     *            输入框背景，见示例中的drawable示例
     * @param passLength
     *            密码长度，根据密码长度，生成相应的输入个数
     * @param passColor
     *            圆圈密码颜色
     * @param passCircleRadius
     *            圆圈密码半径大小,单位：dip
     * @param splitLineWidth
     *            分割线宽度，根据backgroundDrawable边框宽度设置,单位：dip
     * @param splitLineColor
     *            分割线颜色，根据backgroundDrawable边框颜色设置
     */
    private void initStyle(int backgroundDrawable, int passLength, int passColor, int passCircleRadius,
            float splitLineWidth, int splitLineColor) {
        this.inputNumber = passLength;
        // initEdit(backgroundDrawable);
        initShowInput(backgroundDrawable, passColor, passCircleRadius, splitLineWidth, splitLineColor);
    }

    /**
     * @param attrSet
     * @param finishCallBack
     * @param backgroundDrawable
     */
    private void initEdit(CEditTextAttrSet attrSet, CKeyBoardFinishCallBack finishCallBack, int backgroundDrawable) {

        cEditTextView = new CEditTextView(context);
        cEditTextView.initialize(attrSet, finishCallBack);

        cEditTextView.setMaxLength((short) inputNumber);
        cEditTextView.setBackgroundResource(backgroundDrawable);

        cEditTextView.setTextSize(0);
        cEditTextView.addTextChangedListener(new TextWatcher() {

            public void beforeTextChanged(CharSequence s, int start, int count, int after) {
            }

            public void onTextChanged(CharSequence s, int start, int before, int count) {
            }

            public void afterTextChanged(Editable s) {
                initDatas(s);
            }
        });

        LayoutParams editParams = new LayoutParams(LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT);
        editParams.addRule(RelativeLayout.CENTER_IN_PARENT, RelativeLayout.TRUE);
        addView(cEditTextView, editParams);

    }

    private void initShowInput(int backgroundDrawable, int passColor, int passCircleRadius, float splitLineWidth,
            int splitLineColor) {

        linearLayout = new LinearLayout(context);
        linearLayout.setBackgroundResource(backgroundDrawable);
        LayoutParams layoutParams = new LayoutParams(LayoutParams.MATCH_PARENT, LayoutParams.MATCH_PARENT);
        linearLayout.setLayoutParams(layoutParams);
        linearLayout.setOrientation(LinearLayout.HORIZONTAL);
        addView(linearLayout);

        circleViews = new CircleView[inputNumber];
        LinearLayout.LayoutParams circleParams = new LinearLayout.LayoutParams(0, LayoutParams.MATCH_PARENT);
        circleParams.weight = 1;
        circleParams.gravity = Gravity.CENTER;

        LinearLayout.LayoutParams splitParams = new LinearLayout.LayoutParams(dip2px(context, splitLineWidth),
                LayoutParams.MATCH_PARENT);

        passCircleRadius = dip2px(context, passCircleRadius);

        for (int i = 0; i < circleViews.length; i++) {
            LinearLayout layout = new LinearLayout(context);
            layout.setLayoutParams(layoutParams);

            CircleView circleView = new CircleView(context);
            circleView.setCircleRadius(passCircleRadius);
            circleView.setCirCleColor(passColor);
            circleView.setVisibility(View.INVISIBLE);

            circleViews[i] = circleView;
            linearLayout.addView(circleView, circleParams);

            if (i < circleViews.length - 1) {
                View view = new View(context);
                view.setBackgroundColor(splitLineColor);
                linearLayout.addView(view, splitParams);
            }
        }
    }

    /**
     * 根据输入字符，显示密码个数
     *
     * @param s
     */
    public void initDatas(Editable s) {

        if (s.length() == 0) {
            for (int i = 0; i < inputNumber; i++) {
                circleViews[i].setVisibility(View.INVISIBLE);
            }

            return;
        }

        int length = s.length();
        for (int i = 0; i < inputNumber; i++) {
            if (i < length) {
                circleViews[i].setVisibility(View.VISIBLE);
            } else {
                circleViews[i].setVisibility(View.INVISIBLE);
            }
        }
    }

    public static int dip2px(Context context, float dipValue) {
        final float scale = context.getResources().getDisplayMetrics().density;
        return (int) (dipValue * scale + 0.5f);
    }

    public void onDestroy() {
        cEditTextView.onDestroy();
    }

    public String getValue(String timeStamp) throws Exception {
        return cEditTextView.getValue(timeStamp);
    }

    public String getPinValue(String timeStamp) throws Exception {
        return cEditTextView.getPinValue(timeStamp);
    }

    public boolean publicKeyModulus(String keyModulus) {
        return cEditTextView.publicKeyModulus(keyModulus);
    }

    public boolean publicKeyDER(String keyDer) {
        return cEditTextView.publicKeyDER(keyDer);
    }

    public boolean publicKeyBlob(String keyBlob) {
        return cEditTextView.publicKeyBlob(keyBlob);
    }

    public boolean publicKeyAppModulus(String keyModulus) {
        return cEditTextView.publicKeyAppModulus(keyModulus);
    }

    public boolean publicKeyAppDER(String keyDer) {
        return cEditTextView.publicKeyAppDER(keyDer);
    }

    public boolean publicKeyAppBlob(String keyBlob) {
        return cEditTextView.publicKeyAppBlob(keyBlob);
    }

    public boolean publicKeyECC(String x, String y) {
        return cEditTextView.publicKeyECC(x, y);
    }

    public short verify() {
        return cEditTextView.verify();
    }

    public void clear() {
        cEditTextView.clear();
    }

    public short getLength() {
        return cEditTextView.getLength();
    }

    public long getVersion() {
        return cEditTextView.getVersion();
    }

    public char getComplexDegree() {
        return cEditTextView.getComplexDegree();
    }

    public String lastError() {
        return cEditTextView.lastError();
    }

    // public void openCKbd() {
    // cEditTextView.openCKbd();
    // }
    //
    // public void closeCKbd() {
    // cEditTextView.closeCKbd();
    // }

    public void setChallengeCode(byte[] code) {
        cEditTextView.setChallengeCode(code);
    }

    public byte[] getEncryptedPinCode() {
        return cEditTextView.getEncryptedPinCode();
    }

    public void setCalcFactor(String factor) {
        cEditTextView.setCalcFactor(factor);
    }

    public void setContentType(short contentType) {
        cEditTextView.setContentType(contentType);
    }

    public void setSoftkbdType(short softkbdType) {
        cEditTextView.setSoftkbdType(softkbdType);
    }

    public void setSoftkbdStype(short softkbdStype) {
        cEditTextView.setSoftkbdStype(softkbdStype);
    }

    public void setSoftkbdMode(short softkbdMode) {
        cEditTextView.setSoftkbdMode(softkbdMode);
    }

    public void setAccepts(String accepts) {
        cEditTextView.setAccepts(accepts);
    }

    public void setMaxLength(short maxlength) {
        cEditTextView.setMaxLength(maxlength);
    }

    public void setMinLength(short minlength) {
        cEditTextView.setMinLength(minlength);
    }

    public void setMode(short mode) {
        cEditTextView.setMode(mode);
    }

    public void setAlgorithmCode(String algorithmCode) {
        cEditTextView.setAlgorithmCode(algorithmCode);
    }

    public void setHashRandom(String salt) {
        cEditTextView.setHashRandom(salt);
    }

    public void setDictionaryFilter(String dictFilter) {
        cEditTextView.setDictionaryFilter(dictFilter);
    }

    // public String getReceiverAction() {
    // return cEditTextView.getReceiverAction();
    // }

    public String getMeasureValue() {
        return cEditTextView.getMeasureValue();
    }

    public void setFinishMode(short finishMode) {
        cEditTextView.setFinishMode(finishMode);
    }
}
